#include "mozCellMLVariable.h"
#include "mozCellMLVariableResolution.h"
#include "MathMLUtilities.h"
#include <nsIArray.h>
#include <nsArray.h>
#include <nsInterfaceHashtable.h>
#include <nsPrintfCString.h>

#define COMPLAIN(model,args) \
{\
  char* result = PR_smprintf args; \
  nsCAutoString complainCString(result); \
  nsAutoString complainString; \
  CopyUTF8toUTF16(complainCString, complainString); \
  PR_Free(result); \
  return model->ValidationError(complainString); \
}
#define T(model, s) (model->Translate(s))

#define IFOP(n) else if (lname.Equals(NS_LITERAL_STRING(#n)))
#define FUNCCONST(n,v) IFOP(n)\
                         code += NS_LITERAL_CSTRING(v);

#define ROOTCONST(n,v)  else if (name.Equals(NS_LITERAL_STRING(#n)))\
                          code += NS_LITERAL_CSTRING(v);

#define FUNCOP1(n,f) IFOP(n){\
                      if (numSubs != 2)\
                      {\
                        COMPLAIN(model, (T(model, "applyWrongChildren2"), #n, numSubs));\
                        return NS_ERROR_FAILURE;\
                      }\
                      code += NS_LITERAL_CSTRING(#f "("); \
                      e->GetNext(getter_AddRefs(opSupports)); \
                      op = do_QueryInterface(opSupports); \
                      ret = MathMLExpressionAsString(model, op, allVariables, importNumbers, allUnits, component, code, functionStack); \
                      if (ret != NS_OK) \
                        return ret; \
                      code += NS_LITERAL_CSTRING(")"); \
                    }
#define FUNCOP2(n,f) IFOP(n){\
                      if (numSubs != 3)\
                      {\
                        COMPLAIN(model, (T(model, "applyWrongChildren3"),  #n, numSubs));\
                        return NS_ERROR_FAILURE;\
                      }\
                      code += NS_LITERAL_CSTRING(#f "(");\
                      e->GetNext(getter_AddRefs(opSupports)); \
                      op = do_QueryInterface(opSupports); \
                      ret = MathMLExpressionAsString(model, op, allVariables, importNumbers, allUnits, component, code, functionStack); \
                      if (ret != NS_OK) \
                        return ret; \
                      code += ", "; \
                      e->GetNext(getter_AddRefs(opSupports)); \
                      op = do_QueryInterface(opSupports); \
                      ret = MathMLExpressionAsString(model, op, allVariables, importNumbers, allUnits, component, code, functionStack); \
                      if (ret != NS_OK) \
                        return ret; \
                      code += NS_LITERAL_CSTRING(")"); \
                     }
#define FUNCROP1(n,f) IFOP(n){\
                      if (numSubs != 2)\
                      {\
                        COMPLAIN(model, (T(model, "applyWrongChildren2"), #n, \
                                         numSubs)); \
                        return NS_ERROR_FAILURE;\
                      }\
                      code += NS_LITERAL_CSTRING("1 / " #f "(");\
                      e->GetNext(getter_AddRefs(opSupports)); \
                      op = do_QueryInterface(opSupports); \
                      ret = MathMLExpressionAsString(model, op, allVariables, \
                                                     importNumbers, allUnits, \
                                                     component, code, \
                                                     functionStack); \
                      if (ret != NS_OK) \
                        return ret; \
                      code += NS_LITERAL_CSTRING(")"); \
                    }
#define FUNCOPR1(n,f) IFOP(n){\
                      if (numSubs != 2)\
                      {\
                        COMPLAIN(model, (T(model, "applyWrongChildren2"), #n, numSubs));\
                        return NS_ERROR_FAILURE;\
                      }\
                      code += NS_LITERAL_CSTRING(#f "(1 / (");\
                      e->GetNext(getter_AddRefs(opSupports)); \
                      op = do_QueryInterface(opSupports); \
                      ret = MathMLExpressionAsString(model, op, allVariables, importNumbers, allUnits, component, code, functionStack); \
                      if (ret != NS_OK) \
                        return ret; \
                      code += NS_LITERAL_CSTRING("))"); \
                    }
#define COPERN(n,o) IFOP(n){\
                    if (numSubs < 2)\
                    {\
                      COMPLAIN(model, (T(model, "applyWrongChildren2P"), #n, numSubs));\
                      return NS_ERROR_FAILURE;\
                    }\
                    code += NS_LITERAL_CSTRING("(");\
                    e->GetNext(getter_AddRefs(opSupports)); \
                    op = do_QueryInterface(opSupports); \
                    ret = MathMLExpressionAsString(model, op, allVariables, \
                                                   importNumbers, allUnits, \
                                                   component, code, \
                                                   functionStack); \
                    if (ret != NS_OK) \
                      return ret; \
                    code += NS_LITERAL_CSTRING(")"); \
                    for (PRUint32 x = 2; x < numSubs; x++) \
                    { \
                      code += NS_LITERAL_CSTRING(o "("); \
                      e->GetNext(getter_AddRefs(opSupports)); \
                      op = do_QueryInterface(opSupports); \
                      ret = MathMLExpressionAsString(model, op, allVariables, importNumbers, allUnits, component, code, functionStack); \
                      if (ret != NS_OK) \
                        return ret; \
                      code += NS_LITERAL_CSTRING(")"); \
                    } \
                  }
#define COPER12(n,o) IFOP(n){\
                    if (numSubs != 2 && numSubs != 3)\
                    {\
                      COMPLAIN(model, (T(model, "applyWrongChildren23"), #n, numSubs));\
                      return NS_ERROR_FAILURE;\
                    }\
                    if (numSubs == 2)\
                      code += NS_LITERAL_CSTRING(o);\
                    code += NS_LITERAL_CSTRING("(");\
                    e->GetNext(getter_AddRefs(opSupports)); \
                    op = do_QueryInterface(opSupports); \
                    ret = MathMLExpressionAsString(model, op, allVariables, importNumbers, allUnits, component, code, functionStack); \
                    if (ret != NS_OK) \
                      return ret; \
                    code += NS_LITERAL_CSTRING(")"); \
                    if (numSubs == 3)\
                    { \
                      code += NS_LITERAL_CSTRING(o "("); \
                      e->GetNext(getter_AddRefs(opSupports)); \
                      op = do_QueryInterface(opSupports); \
                      ret = MathMLExpressionAsString(model, op, allVariables, importNumbers, allUnits, component, code, functionStack); \
                      if (ret != NS_OK) \
                        return ret; \
                      code += NS_LITERAL_CSTRING(")"); \
                    } \
                  }
#define COPER1(n,o) IFOP(n){\
                    if (numSubs != 2)\
                    {\
                      COMPLAIN(model, (T(model, "applyWrongChildren2"), #n, numSubs));\
                      return NS_ERROR_FAILURE;\
                    }\
                    code += NS_LITERAL_CSTRING("!(");\
                    e->GetNext(getter_AddRefs(opSupports)); \
                    op = do_QueryInterface(opSupports); \
                    ret = MathMLExpressionAsString(model, op, allVariables, importNumbers, allUnits, component, code, functionStack); \
                    if (ret != NS_OK) \
                      return ret; \
                    code += NS_LITERAL_CSTRING(")"); \
                  }
#define FUNCDEFAULT(n,f,d,m,o) IFOP(n){\
      if (numSubs < 2 || numSubs > 3)\
      {\
        COMPLAIN(model, (T(model, "applyWrongChildren23"), #n, numSubs));\
        return NS_ERROR_FAILURE;\
      }\
      nsCOMPtr<nsIDOMElement> inside;\
      if (numSubs == 2)\
      {\
        nsCOMPtr<nsISupports> insideSupports; \
        e->GetNext(getter_AddRefs(insideSupports));\
        inside = do_QueryInterface(insideSupports); \
        code += #f "(";\
        ret = MathMLExpressionAsString(model, inside, allVariables, importNumbers, allUnits, component, code, functionStack);\
        if (ret != NS_OK) \
          return ret; \
        code += ", " #d ")"; \
      }\
      else\
      {\
        nsCOMPtr<nsISupports> p1Supports, p2Supports; \
        e->GetNext(getter_AddRefs(p1Supports)); \
        e->GetNext(getter_AddRefs(p2Supports)); \
        nsCOMPtr<nsIDOMElement> p1(do_QueryInterface(p1Supports)), \
                                p2(do_QueryInterface(p2Supports)), degV; \
        nsAutoString nm1, nm2;\
        p1->GetLocalName(nm1);\
        p2->GetLocalName(nm2);\
        if (nm1.Equals(NS_LITERAL_STRING(#o)))\
        {\
          inside = p2;\
          degV = p1;\
        }\
        else if (nm2.Equals(NS_LITERAL_STRING(#o)))\
        {\
          inside = p1;\
          degV = p2;\
        }\
        else\
        {\
          COMPLAIN(model, (T(model, "malformedFuncInvalidChild"), #n, #o));\
          return NS_ERROR_FAILURE;\
        }\
        nsCOMPtr<nsIDOMNodeList> degSubnodesAll;\
        nsCOMPtr<nsIArray> degSubnodes;\
        degV->GetChildNodes(getter_AddRefs(degSubnodesAll));\
        FilterNodeListByTypeNS(degSubnodesAll, getter_AddRefs(subnodes),\
                               nsIDOMNode::ELEMENT_NODE, MATHML_NS);\
        PRUint32 numDegs;\
        degSubnodes->GetLength(&numDegs);\
        if (numDegs != 1)\
        {\
          COMPLAIN(model, (T(model, "malformedFuncWrongOverride"), #n, #m, numDegs));\
          return NS_ERROR_FAILURE;\
        }\
        nsCOMPtr<nsISimpleEnumerator> de;\
        degSubnodes->Enumerate(getter_AddRefs(de));\
        nsCOMPtr<nsISupports> ddeSupports; \
        de->GetNext(getter_AddRefs(ddeSupports)); \
        nsCOMPtr<nsIDOMElement> dde(do_QueryInterface(ddeSupports)); \
        code += #f "(";\
        ret = MathMLExpressionAsString(model, inside, allVariables, importNumbers, allUnits, component, code, functionStack);\
        if (ret != NS_OK) \
          return ret; \
        code += ", " m "(";\
        ret = MathMLExpressionAsString(model, dde, allVariables, importNumbers, allUnits, component, code, functionStack);\
        if (ret != NS_OK) \
          return ret; \
        code += "))";\
      } \
                             }

static nsresult
FindUnits(
          mozCellML* model,
          nsIDOMNode* compNode,
          nsDataHashtable<nsISupportsHashKey, PRUint32>&
            importNumbers,
          nsInterfaceHashtable<nsStringHashKey,
                               mozCellMLUnits>& allUnits,
          nsAutoString& unitsName,
          mozCellMLUnits** foundUnits
         )
{
  nsCOMPtr<nsIDOMElement> component(do_QueryInterface(compNode));
  nsCOMPtr<nsIDOMNode> modelNode;
  component->GetParentNode(getter_AddRefs(modelNode));
  nsCOMPtr<nsIDOMElement> modelEl(do_QueryInterface(modelNode));

  PRUint32 impNo;
  importNumbers.Get(modelEl, &impNo);

#ifdef DEBUG_UNITS
  printf("Setting up variable units for import # %u\n", impNo);
#endif

  nsPrintfCString impNoCStr("%u:", impNo);

  nsAutoString impNoStr;
  CopyUTF8toUTF16(impNoCStr, impNoStr);
  
  nsAutoString compName;
  component->GetAttribute(NS_LITERAL_STRING("name"), compName);

  // Attempt 1: In the component...
  nsAutoString findAttempt = impNoStr + compName + NS_LITERAL_STRING(":") +
                             unitsName;
  allUnits.Get(findAttempt, foundUnits);
  if (*foundUnits != nsnull)
    return NS_OK;

  // Attempt 2: In the same model(or imported).
  findAttempt = impNoStr + NS_LITERAL_STRING(":") + unitsName;
  allUnits.Get(findAttempt, foundUnits);
  if (*foundUnits != nsnull)
    return NS_OK;

  // Attempt 3: Built in...
  allUnits.Get(unitsName, foundUnits);
  if (*foundUnits != nsnull)
    return NS_OK;

  nsCAutoString unitsNameC;
  nsCAutoString compNameC;
  CopyUTF16toUTF8(unitsName, unitsNameC);
  CopyUTF16toUTF8(compName, compNameC);
  COMPLAIN(model, (T(model, "cantFindUnits"), unitsNameC.get(),
                   compNameC.get()));
}

nsresult
MathMLExpressionAsString(
                         mozCellML* model,
                         nsIDOMElement* from,
                         nsInterfaceHashtable<nsStringHashKey,
                                              mozCellMLVariableResolution>&
                           allVariables,
                         nsDataHashtable<nsISupportsHashKey,PRUint32>&
                           importNumbers,
                         nsInterfaceHashtable<nsStringHashKey,
                                              mozCellMLUnits>&
                           allUnits,
                         nsIDOMElement* component,
                         nsACString& code,
                         nsCStringArray& functionStack
                        )
{
  nsAutoString name;
  from->GetLocalName(name);
  if (name.Equals(NS_LITERAL_STRING("ci")))
  {
    nsCOMPtr<nsIDOMNodeList> subNodes;
    from->GetChildNodes(getter_AddRefs(subNodes));
    nsCOMPtr<nsIArray> subText;
    FilterNodeListByType(subNodes, getter_AddRefs(subText),
                         nsIDOMNode::TEXT_NODE);
    PRUint32 numText;
    subText->GetLength(&numText);
    if (numText == 0)
    {
      COMPLAIN(model, (T(model, "noTextNodesInCI")))
    }

    nsCOMPtr<nsISimpleEnumerator> subTextE;
    subText->Enumerate(getter_AddRefs(subTextE));
    nsCOMPtr<nsISupports> txtNodeSupports;
    subTextE->GetNext(getter_AddRefs(txtNodeSupports));
    nsCOMPtr<nsIDOMText> txt(do_QueryInterface(txtNodeSupports));
    nsAutoString nameStr;
    txt->GetData(nameStr);
    nameStr.Trim(" \t\r\n\f");

    nsCOMPtr<mozCellMLVariableResolution> v;

    nsAutoString impName;
    
    PRUint32 impNo;
    nsCOMPtr<nsIDOMNode> modelNode;
    component->GetParentNode(getter_AddRefs(modelNode));
    nsCOMPtr<nsIDOMElement> modelEl(do_QueryInterface(modelNode));
    importNumbers.Get(modelEl, &impNo);
    nsPrintfCString impNoCStr("%u:", impNo);
    nsAutoString impNoStr;
    CopyUTF8toUTF16(impNoCStr, impNoStr);

    nsAutoString componentName;
    component->GetAttribute(NS_LITERAL_STRING("name"), componentName);

    allVariables.Get(impNoStr + componentName + NS_LITERAL_STRING(":") +
                     nameStr, getter_AddRefs(v));
    
    if (v == nsnull)
    {
      nsCAutoString cname, cassignComponent;
      CopyUTF16toUTF8(nameStr, cname);
      CopyUTF16toUTF8(componentName, cassignComponent);
      COMPLAIN(model, (T(model, "cantFindVariableInComponent"),
                       cname.get(), cassignComponent.get()))
    }
    mozCellMLVariable* uv = v->GetVariable();

    if (uv->type == mozICellMLVariable::SIMPLE ||
        uv->type == mozICellMLVariable::DIFFERENTIAL ||
        uv->type == mozICellMLVariable::CONSTANT ||
        uv->type == mozICellMLVariable::BOUND)
    {
      nsCAutoString cname;
      v->GetVariableRHS(cname);
      code += cname;
    }
    else
      COMPLAIN(model, ("Bug?: Unknown variable type %u!!!\n", uv->type))
    return NS_OK;
  }
  else if (name.Equals(NS_LITERAL_STRING("cn")))
  {
    double value;
    nsCOMPtr<nsIDOMNodeList> subNodes;
    from->GetChildNodes(getter_AddRefs(subNodes));
    nsCOMPtr<nsIArray> subText;
    FilterNodeListByType(subNodes, getter_AddRefs(subText),
                         nsIDOMNode::TEXT_NODE);
    PRUint32 numText;
    subText->GetLength(&numText);
    if (numText == 0)
      return NS_ERROR_FAILURE;
    nsCOMPtr<nsISimpleEnumerator> subTextE;
    subText->Enumerate(getter_AddRefs(subTextE));
    nsCOMPtr<nsISupports> txtNodeSupports;
    subTextE->GetNext(getter_AddRefs(txtNodeSupports));
    nsCOMPtr<nsIDOMText> txt(do_QueryInterface(txtNodeSupports));
    nsAutoString valueStr;
    nsCAutoString valueCStr;
    txt->GetData(valueStr);
    valueStr.Trim(" \t\r\n\f");
    CopyUTF16toUTF8(valueStr, valueCStr);
    value = PR_strtod(valueCStr.get(), nsnull);

    nsPrintfCString lit("%e", value);
    code += lit;
    nsAutoString unitsName;
    if ((
         (from->GetAttributeNS(CELLML_NS_1_0, NS_LITERAL_STRING("units"),
                               unitsName) == NS_OK) &&
         !unitsName.Equals(NS_LITERAL_STRING(""))
        ) ||
        (
         (from->GetAttributeNS(CELLML_NS_1_1, NS_LITERAL_STRING("units"),
                               unitsName) == NS_OK) &&
         !unitsName.Equals(NS_LITERAL_STRING(""))
        )
       )
    {
      nsCOMPtr<mozCellMLUnits> cnUnit;
      nsresult ret = FindUnits(model, component, importNumbers, allUnits,
                               unitsName, getter_AddRefs(cnUnit));
      NS_ENSURE_SUCCESS(ret, ret);
    }
    else
      COMPLAIN(model, (T(model, "cnMissingUnits")));
  }
  else if (name.Equals(NS_LITERAL_STRING("piecewise")))
  {
    nsresult ret;
    // This way of doing things is short but ugly...
    // Note that we assume otherwise is last, otherwise we ignore the rest...
    nsCOMPtr<nsIDOMNodeList> subnodesAll;
    nsCOMPtr<nsIArray> subnodes;
    from->GetChildNodes(getter_AddRefs(subnodesAll));
    FilterNodeListByTypeNS(subnodesAll, getter_AddRefs(subnodes),
                           nsIDOMNode::ELEMENT_NODE, MATHML_NS);
    PRUint32 numSubs;
    PRBool gotOtherwise = PR_FALSE;
    subnodes->GetLength(&numSubs);
    if (numSubs < 1)
    {
      COMPLAIN(model, (T(model, "picewiseNoChildren")));
    }
    nsCOMPtr<nsISimpleEnumerator> e;
    subnodes->Enumerate(getter_AddRefs(e));
    nsCOMPtr<nsIDOMElement> piece;
    for (PRUint32 n = 0; n < numSubs; n++)
    {
      nsCOMPtr<nsISupports> pieceSupports;
      e->GetNext(getter_AddRefs(pieceSupports));
      piece = do_QueryInterface(pieceSupports);
      nsCOMPtr<nsIDOMNodeList> psubnodesAll;
      nsCOMPtr<nsIArray> psubnodes;
      piece->GetChildNodes(getter_AddRefs(psubnodesAll));
      FilterNodeListByTypeNS(psubnodesAll, getter_AddRefs(psubnodes),
                             nsIDOMNode::ELEMENT_NODE, MATHML_NS);
      PRUint32 numPSubs;
      psubnodes->GetLength(&numPSubs);
      nsCOMPtr<nsISimpleEnumerator> pe;
      psubnodes->Enumerate(getter_AddRefs(pe));
      nsAutoString pieceType;
      piece->GetLocalName(pieceType);
      if (pieceType.Equals(NS_LITERAL_STRING("piece")))
      {
        if (numPSubs != 2)
          COMPLAIN(model, (T(model, "pieceWrongChildren"), numPSubs))
        nsCOMPtr<nsISupports> valueSupports, conditionSupports;
        pe->GetNext(getter_AddRefs(valueSupports));
        pe->GetNext(getter_AddRefs(conditionSupports));
        nsCOMPtr<nsIDOMElement> value(do_QueryInterface(valueSupports)),
                                condition(do_QueryInterface(conditionSupports));
        code += "(";
        ret = MathMLExpressionAsString(model, condition, allVariables,
                                       importNumbers, allUnits, component, code,
                                       functionStack);
        if (ret != NS_OK)
          return ret;
        code += ") ? (";
        ret = MathMLExpressionAsString(model, value, allVariables, importNumbers,
                                       allUnits, component, code, functionStack);
        if (ret != NS_OK)
          return ret;
        code += ") : ";
        if (ret != NS_OK)
          return ret;
      }
      else if (pieceType.Equals(NS_LITERAL_STRING("otherwise")))
      {
        gotOtherwise = PR_TRUE;
        if (numPSubs != 1)
          COMPLAIN(model, (T(model, "otherwiseWrongChildren"), numPSubs))
        nsCOMPtr<nsISupports> valueSupports;
        pe->GetNext(getter_AddRefs(valueSupports));
        nsCOMPtr<nsIDOMElement> value(do_QueryInterface(valueSupports));
        ret = MathMLExpressionAsString(model, value, allVariables,
                                       importNumbers, allUnits, component,
                                       code, functionStack);
        if (ret != NS_OK)
          return ret;
        break;
      }
    }
    if (!gotOtherwise)
      code += "nan_value";
  }
  else if (name.Equals(NS_LITERAL_STRING("apply")))
  {
    nsCOMPtr<nsIDOMNodeList> subnodesAll;
    nsCOMPtr<nsIArray> subnodes;
    from->GetChildNodes(getter_AddRefs(subnodesAll));
    FilterNodeListByTypeNS(subnodesAll, getter_AddRefs(subnodes),
                           nsIDOMNode::ELEMENT_NODE, MATHML_NS);
    PRUint32 numSubs;
    subnodes->GetLength(&numSubs);
    if (numSubs < 1)
      COMPLAIN(model, (T(model, "applyWithNoElem")))
    nsCOMPtr<nsISimpleEnumerator> e;
    subnodes->Enumerate(getter_AddRefs(e));
    nsCOMPtr<nsISupports> opSupports;
    e->GetNext(getter_AddRefs(opSupports));
    nsCOMPtr<nsIDOMElement> op(do_QueryInterface(opSupports));
    nsAutoString lname;
    op->GetLocalName(lname);
    nsresult ret;
    // We now have to look at the operator, and convert to a C operator...
    if (lname.Equals(NS_LITERAL_STRING("quotient")))
    {
      if (numSubs != 3)
        COMPLAIN(model, (T(model, "quotientWrongChildren"),numSubs))
      code += "floor(";
      e->GetNext(getter_AddRefs(opSupports));
      op = do_QueryInterface(opSupports);
      ret = MathMLExpressionAsString(model, op, allVariables, importNumbers,
                                     allUnits, component, code, functionStack);
      if (ret != NS_OK)
        return ret;
      code += "/";
      e->GetNext(getter_AddRefs(opSupports));
      op = do_QueryInterface(opSupports);
      ret = MathMLExpressionAsString(model, op, allVariables, importNumbers,
                                     allUnits, component, code, functionStack);
      if (ret != NS_OK)
        return ret;
      code += ")";
    }
    FUNCCONST(pi, "3.14159265358979323846")
    FUNCCONST(exponentiale, "2.7182818284590452354")
    FUNCCONST(eulergamma, "0.5772156649")
    FUNCCONST(infinity, "infinity()")
    FUNCCONST(notanumber, "nan()")
    FUNCCONST(true, "1.0")
    FUNCCONST(false, "0.0")
    COPERN(divide, "/")
    COPER12(minus, "-")
    COPERN(plus, "+")
    COPERN(times, "*")
    COPERN(and, "&&")
    COPERN(or, "||")
    COPER1(not, "!")
    COPERN(eq, "==")
    COPERN(neq, "!=")
    COPERN(gt, ">")
    COPERN(lt, "<")
    COPERN(geq, ">=")
    COPERN(leq, "<=")
    FUNCOP2(xor, logical_xor)
    FUNCOP2(implies, logical_implies)
    FUNCOP1(factorial, factorial)
    FUNCOP1(exp, exp)
    FUNCOP2(power, pow)
    FUNCDEFAULT(root, pow, 0.5, "1/", degree)
    FUNCOP1(floor, floor)
    FUNCOP1(ceiling, ceil)
    FUNCOP1(abs, abs_d)
    FUNCOP1(ln, log)
    FUNCDEFAULT(log, arbitrary_log, 10, "", logbase)
    FUNCOP2(rem, get_remainder)
    FUNCOP1(sin, sin)
    FUNCOP1(cos, cos)
    FUNCOP1(tan, tan)
    FUNCROP1(csc, sin)
    FUNCROP1(sec, cos)
    FUNCROP1(cot, tan)
    FUNCOP1(sinh, sinh)
    FUNCOP1(cosh, cosh)
    FUNCOP1(tanh, tanh)
    FUNCROP1(csch, sinh)
    FUNCROP1(sech, cosh)
    FUNCROP1(coth, tanh)
    FUNCOP1(arcsin, arcsin)
    FUNCOP1(arccos, arccos)
    FUNCOP1(arctan, arctan)
    FUNCOP1(arccosh, arccosh)
    FUNCOPR1(arccot, arctan)
    FUNCOPR1(arccoth, arctanh)
    FUNCOPR1(arccsc, arcsin)
    FUNCOPR1(arccsch, arcsinh)
    FUNCOPR1(arcsec, arccos)
    FUNCOPR1(arcsech, arccosh)
    FUNCOP1(arcsinh, arcsinh)
    FUNCOP1(arctanh, arctanh)
    else if (lname.Equals(NS_LITERAL_STRING("int")))
    {
      nsCOMPtr<nsIDOMNodeList> subnodesAll;
      from->GetChildNodes(getter_AddRefs(subnodesAll));
      // Look for upper bounds, lower bounds, and the value to compute...
      nsCOMPtr<nsIArray> mathml;
      nsCOMPtr<nsIDOMElement> integrate, uplimit, lowlimit, intbvar;
      FilterNodeListByTypeNS(subnodesAll, getter_AddRefs(mathml),
                             nsIDOMElement::ELEMENT_NODE, MATHML_NS);
      PRUint32 i, l;
      nsCOMPtr<nsISimpleEnumerator> e;
      mathml->GetLength(&l);
      mathml->Enumerate(getter_AddRefs(e));
      for (i = 0; i < l; i++)
      {
        nsCOMPtr<nsISupports> nSupports;
        e->GetNext(getter_AddRefs(nSupports));
        if (i == 0)
          continue;
        nsCOMPtr<nsIDOMElement> n(do_QueryInterface(nSupports));
        nsAutoString nname;
        n->GetLocalName(nname);
        // interval, condition
        if (nname.Equals(NS_LITERAL_STRING("lowlimit")))
        {
          if (lowlimit != nsnull)
            COMPLAIN(model, (T(model, "lowlimitRepeatedInInt")))
          n->GetChildNodes(getter_AddRefs(subnodesAll));
          nsCOMPtr<nsIArray> unodes;
          FilterNodeListByTypeNS(subnodesAll,
                                 getter_AddRefs(unodes),
                                 nsIDOMNode::ELEMENT_NODE,
                                 MATHML_NS);
          PRUint32 nuels;
          unodes->GetLength(&nuels);
          if (nuels != 1)
            COMPLAIN(model, (T(model, "lowlimitWrongChildren"), nuels));
          unodes->QueryElementAt(0,
                                 NS_GET_IID(nsIDOMElement),
                                 getter_AddRefs(lowlimit));
        }
        else if (nname.Equals(NS_LITERAL_STRING("uplimit")))
        {
          if (uplimit != nsnull)
            COMPLAIN(model, (T(model, "uplimitRepeatedInInt")))
          n->GetChildNodes(getter_AddRefs(subnodesAll));
          nsCOMPtr<nsIArray> unodes;
          FilterNodeListByTypeNS(subnodesAll,
                                 getter_AddRefs(unodes),
                                 nsIDOMNode::ELEMENT_NODE,
                                 MATHML_NS);
          PRUint32 nuels;
          unodes->GetLength(&nuels);
          if (nuels != 1)
            COMPLAIN(model, (T(model, "uplimitWrongChildren"), nuels));
          unodes->QueryElementAt(0,
                                 NS_GET_IID(nsIDOMElement),
                                 getter_AddRefs(uplimit));
        }
        else if (nname.Equals(NS_LITERAL_STRING("domainofapplication")))
        {
          // Do nothing...
        }
        else if (nname.Equals(NS_LITERAL_STRING("bvar")))
        {
          if (intbvar != nsnull)
            COMPLAIN(model, (T(model, "bvarRepeatedInInt")))
          intbvar = n;
        }
        else if (nname.Equals(NS_LITERAL_STRING("condition")))
          COMPLAIN(model, (T(model, "dontSupportCondition")))
        else if (nname.Equals(NS_LITERAL_STRING("interval")))
        {
          if (lowlimit != nsnull)
            COMPLAIN(model, (T(model, "lowlimitRepeatedInInt")))
          if (uplimit != nsnull)
            COMPLAIN(model, (T(model, "uplimitRepeatedInInt")))
          // Fetch the two nodes...
          n->GetChildNodes(getter_AddRefs(subnodesAll));
          nsCOMPtr<nsIArray> intvNodes;
          FilterNodeListByTypeNS(subnodesAll, getter_AddRefs(intvNodes),
                                 nsIDOMElement::ELEMENT_NODE, MATHML_NS);
          PRUint32 vl;
          intvNodes->GetLength(&vl);
          if (vl != 2)
            COMPLAIN(model, (T(model, "wrongChildrenInterval"), vl));
          intvNodes->QueryElementAt(0, NS_GET_IID(nsIDOMElement),
                                    getter_AddRefs(lowlimit));
          intvNodes->QueryElementAt(1, NS_GET_IID(nsIDOMElement),
                                    getter_AddRefs(uplimit));
        }
        else
        {
          if (integrate != nsnull)
          {
            nsAutoString nname2;
            integrate->GetLocalName(nname2);
            nsCAutoString cnname, cnname2;
            CopyUTF16toUTF8(nname, cnname);
            CopyUTF16toUTF8(nname2, cnname2);
            COMPLAIN(model, (T(model, "integrandRepeatedInInt"), cnname.get(),
                             cnname2.get()))
          }
          integrate = n;
        }
      }
      if (lowlimit == nsnull)
        COMPLAIN(model, (T(model, "lowLimitMissingInInt")))
      else if (uplimit == nsnull)
        COMPLAIN(model, (T(model, "upLimitMissingInInt")))
      else if (intbvar == nsnull)
        COMPLAIN(model, (T(model, "bvarMissingInInt")))
      else if (integrate == nsnull)
      {
        // XXX should we pretend it was <cn>1</cn>?
        COMPLAIN(model, (T(model, "integrandMissingInInt")));
      }
      
      nsCOMPtr<mozCellMLVariableResolution> bv;
      nsAutoString bvname;

      nsCOMPtr<nsIDOMNodeList> bvarNodesAll;
      intbvar->GetChildNodes(getter_AddRefs(bvarNodesAll));
      nsCOMPtr<nsIArray> bvarEls;
      FilterNodeListByTypeNS(bvarNodesAll, getter_AddRefs(bvarEls),
                             nsIDOMNode::ELEMENT_NODE,
                             MATHML_NS);
      PRUint32 bvarElsLength;
      bvarEls->GetLength(&bvarElsLength);
      if (bvarElsLength != 1)
        COMPLAIN(model, (T(model, "wrongBvarChildren"), bvarElsLength));
      
      nsCOMPtr<nsIDOMElement> bvarCiEl;
      bvarEls->QueryElementAt(0, NS_GET_IID(nsIDOMElement), getter_AddRefs(bvarCiEl));
      
      // Make sure it really is a <ci>...
      nsAutoString tn;
      bvarCiEl->GetLocalName(tn);
      if (!tn.Equals(NS_LITERAL_STRING("ci")))
      {
        nsCAutoString ctn;
        CopyUTF16toUTF8(tn, ctn);
        COMPLAIN(model, (T(model, "bvarChildNotCi"), ctn.get()));
      }

      nsCOMPtr<nsIDOMNodeList> subNodes;
      bvarCiEl->GetChildNodes(getter_AddRefs(subNodes));
      nsCOMPtr<nsIArray> subText;
      FilterNodeListByType(subNodes, getter_AddRefs(subText),
                           nsIDOMNode::TEXT_NODE);
      PRUint32 numText;
      subText->GetLength(&numText);
      if (numText == 0)
        return NS_ERROR_FAILURE;
      nsCOMPtr<nsISimpleEnumerator> subTextE;
      subText->Enumerate(getter_AddRefs(subTextE));
      nsCOMPtr<nsISupports> txtNodeSupports;
      subTextE->GetNext(getter_AddRefs(txtNodeSupports));
      nsCOMPtr<nsIDOMText> txt(do_QueryInterface(txtNodeSupports));
      nsAutoString bvarNameStr;
      txt->GetData(bvarNameStr);
      bvarNameStr.Trim(" \t\r\n\f");

      nsAutoString componentName;
      component->GetAttribute(NS_LITERAL_STRING("name"), componentName);

      PRUint32 impNo;

      nsCOMPtr<nsIDOMNode> modelNode;
      component->GetParentNode(getter_AddRefs(modelNode));
      nsCOMPtr<nsIDOMElement> modelEl(do_QueryInterface(modelNode));
      importNumbers.Get(modelEl, &impNo);
      nsPrintfCString impNoCStr("%u:", impNo);
      nsAutoString impNoStr;
      CopyUTF8toUTF16(impNoCStr, impNoStr);

      allVariables.Get(impNoStr + componentName + NS_LITERAL_STRING(":") +
                       bvarNameStr, getter_AddRefs(bv));
      // We need to get the variable number for the bound...
      PRInt32 boundno = bv->GetVariable()->GetCode();
      // We also need to get the conversion factors...
      double convfact = bv->GetA();

      nsCAutoString integrand;
      ret = MathMLExpressionAsString(model, integrate, allVariables,
                                     importNumbers, allUnits,
                                     component, integrand,
                                     functionStack);
      if (ret != NS_OK)
        return ret;
      nsCAutoString intf("double function");
      PRUint32 funcno = model->GetFunctionNumber();
      nsPrintfCString funcn("%u", funcno);
      printf("boundno for the variable is %u\n", boundno);
      nsPrintfCString boundn("%d", boundno);
      intf += funcn;
      intf += "(double y, double* x)\n{\n  return \n";
      intf += integrand;
      intf += ";\n}\n";
      functionStack.AppendCString(intf);
      code += "(inthp(";
      ret = MathMLExpressionAsString(model, lowlimit, allVariables,
                                     importNumbers, allUnits, component,
                                     code, functionStack);
      if (ret != NS_OK)
        return ret;
      code += ", ";
      ret = MathMLExpressionAsString(model, uplimit, allVariables,
                                     importNumbers, allUnits, component,
                                     code, functionStack);
      if (ret != NS_OK)
        return ret;
      code += ", &function";
      code += funcn;
      code += ", 1000000, rel_tolerance,&y,x,";
      code += boundn;
      code += ")";
      if (convfact != 1.0)
      {
        nsPrintfCString convfacts("*%e", convfact);
        code += convfacts;
      }
      code += ")";
    }
    else
    {
      nsCAutoString cname;
      CopyUTF16toUTF8(lname, cname);
      COMPLAIN(model, (T(model, "unknownMathMLOperator"), cname.get()))
    }
    return NS_OK;
  }
  ROOTCONST(pi, "3.14159265358979323846")
  ROOTCONST(exponentiale, "2.7182818284590452354")
  ROOTCONST(eulergamma, "0.5772156649")
  ROOTCONST(infinity, "infinity()")
  ROOTCONST(notanumber, "nan()")
  ROOTCONST(true, "1.0")
  ROOTCONST(false, "0.0")
  else
  {
    nsCAutoString fv;
    CopyUTF16toUTF8(name, fv);
    COMPLAIN(model, (T(model, "unknownMathMLOperand"), fv.get()))
    return NS_ERROR_FAILURE;
  }
  return NS_OK;
}
