/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz>
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "mozCellMLChannel.h"
#include <nsIURL.h>
#include <nsCOMPtr.h>
#include <nsString.h>
#include <nsContentCID.h>
#include <nsIStreamListener.h>
#include <nsIWindowWatcher.h>
#include <necko/nsNetCID.h>
#include <necko/nsNetUtil.h>

NS_INTERFACE_MAP_BEGIN(mozCellMLChannel)
  NS_INTERFACE_MAP_ENTRY(nsIChannel)
  NS_INTERFACE_MAP_ENTRY(nsIRequest)
  NS_INTERFACE_MAP_ENTRY(nsIDocumentObserver)
  NS_INTERFACE_MAP_ENTRY(mozCellMLIImportConsumer)
  NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
  NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIChannel)
NS_INTERFACE_MAP_END
NS_IMPL_ADDREF(mozCellMLChannel)
NS_IMPL_RELEASE(mozCellMLChannel)

NS_INTERFACE_MAP_BEGIN(mozCellMLImportRequest)
  NS_INTERFACE_MAP_ENTRY(nsIChannel)
  NS_INTERFACE_MAP_ENTRY(nsIRequest)
  NS_INTERFACE_MAP_ENTRY(nsIDocumentObserver)
  NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
  NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIChannel)
NS_INTERFACE_MAP_END
NS_IMPL_ADDREF(mozCellMLImportRequest)
NS_IMPL_RELEASE(mozCellMLImportRequest)


mozCellMLChannel::mozCellMLChannel(nsIChannel* aChan, PRBool iisvalidator)
  : mChan(aChan), loadCancelled(PR_FALSE), isvalidator(iisvalidator)
{
  LEAKTRACE_CREATED(mozCellMLChannel)
}

mozCellMLChannel::~mozCellMLChannel()
{
  LEAKTRACE_DELETED(mozCellMLChannel)
  if (mDoc != nsnull)
  {
    mDoc->RemoveObserver(NS_STATIC_CAST(nsIDocumentObserver*, this));
    mDoc = nsnull;
  }

}

NS_IMETHODIMP
mozCellMLChannel::GetName(nsACString& name)
{
  return mChan->GetName(name);
}

NS_IMETHODIMP
mozCellMLChannel::IsPending(PRBool* pend)
{
  return mChan->IsPending(pend);
}

NS_IMETHODIMP
mozCellMLChannel::GetStatus(nsresult* stat)
{
  return mChan->GetStatus(stat);
}

NS_IMETHODIMP
mozCellMLChannel::Cancel(nsresult ret)
{
  return mChan->Cancel(ret);
}

NS_IMETHODIMP
mozCellMLChannel::Suspend()
{
  return mChan->Suspend();
}

NS_IMETHODIMP
mozCellMLChannel::Resume()
{
  return mChan->Resume();
}

NS_IMETHODIMP
mozCellMLChannel::SetLoadGroup(nsILoadGroup* aLoadGroup)
{
  NS_ASSERTION(doc == nsnull, "SetLoadGroup called after AsyncOpen()");
  mLoadGroup = aLoadGroup;
  return mChan->SetLoadGroup(aLoadGroup);
}

NS_IMETHODIMP
mozCellMLChannel::GetLoadGroup(nsILoadGroup** aLoadGroup)
{
  return mChan->GetLoadGroup(aLoadGroup);
}

NS_IMETHODIMP
mozCellMLChannel::SetLoadFlags(nsLoadFlags loadFlags)
{
  return mChan->SetLoadFlags(loadFlags);
}

NS_IMETHODIMP
mozCellMLChannel::GetLoadFlags(nsLoadFlags* loadFlags)
{
  return mChan->GetLoadFlags(loadFlags);
}

NS_IMETHODIMP
mozCellMLChannel::GetOriginalURI(nsIURI** aURI)
{
  if (mURI == nsnull)
  {
#ifdef DEBUG_MODE
    printf("Failing because we can't GetOriginalURI!\n");
#endif
    return NS_ERROR_FAILURE;
  }
  *aURI = mURI;
  NS_ADDREF(*aURI);
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLChannel::SetOriginalURI(nsIURI* aURI)
{
  mURI = aURI;
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLChannel::GetURI(nsIURI** aURI)
{
  return mChan->GetURI(aURI);
}

NS_IMETHODIMP
mozCellMLChannel::GetOwner(nsISupports** owner)
{
  return mChan->GetOwner(owner);
}

NS_IMETHODIMP
mozCellMLChannel::SetOwner(nsISupports* owner)
{
  return mChan->SetOwner(owner);
}

NS_IMETHODIMP
mozCellMLChannel::SetNotificationCallbacks(nsIInterfaceRequestor* notcb)
{
  return mChan->SetNotificationCallbacks(notcb);
}

NS_IMETHODIMP
mozCellMLChannel::GetNotificationCallbacks(nsIInterfaceRequestor** notcb)
{
  return mChan->GetNotificationCallbacks(notcb);
}

NS_IMETHODIMP
mozCellMLChannel::GetSecurityInfo(nsISupports** secInfo)
{
  return mChan->GetSecurityInfo(secInfo);
}

NS_IMETHODIMP
mozCellMLChannel::GetContentType(nsACString& ctype)
{
#ifdef DEBUG_MODE
  printf("GetContentType called.\n");
#endif
  ctype.Assign("text/xml");
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLChannel::SetContentType(const nsACString& ctype)
{
  return mChan->SetContentType(ctype);
}

NS_IMETHODIMP
mozCellMLChannel::GetContentCharset(nsACString& ccharset)
{
  return mChan->GetContentCharset(ccharset);
}

NS_IMETHODIMP
mozCellMLChannel::SetContentCharset(const nsACString& ccharset)
{
  return mChan->SetContentCharset(ccharset);
}

NS_IMETHODIMP
mozCellMLChannel::GetContentLength(PRInt32* length)
{
  return mChan->GetContentLength(length);
}

NS_IMETHODIMP
mozCellMLChannel::SetContentLength(PRInt32 length)
{
  return mChan->SetContentLength(length);
}

NS_IMETHODIMP
mozCellMLChannel::Open(nsIInputStream** iis)
{
#ifdef DEBUG_MODE
  printf("Failing because we can't Open()...\n");
#endif
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
mozCellMLChannel::AsyncOpen(
                            nsIStreamListener* aListener,
                            nsISupports* context
                           )
{
  // We ignore listener, as we don't provide data back to the consumer, instead
  // it goes to the document, which eventually gets displayed in a chrome
  // window or similar.

#ifdef DEBUG_MODE
  printf("mozCellMLChannel AsyncOpen()\n");
#endif
  // This is a hack to make the 1.7.x tree work...

  nsresult rv;

  static NS_DEFINE_CID(kXMLDocumentCID,
                       NS_XMLDOCUMENT_CID
                      );

  mDoc = do_CreateInstance("@mozilla.org/xml/xml-document;1", &rv);
#ifdef DEBUG_MODE
  printf("Creating a document...\n");
#endif
  if (mDoc == nsnull)
  {
    printf("Can't create XML document: %u. Trying by CID\n", rv);
    mDoc = do_CreateInstance(kXMLDocumentCID);
  }
  if (mDoc == nsnull)
  {
#ifdef DEBUG_MODE
    printf("mDoc could not be created!!!\n");
#endif
    return NS_ERROR_OUT_OF_MEMORY;
  }

#ifdef DEBUG_MODE
  printf("Starting document load for underlying document.\n");
#endif
  rv = mDoc->StartDocumentLoad(
                               "loadAsData",
                               this,
                               mLoadGroup,
                               nsnull,
                               getter_AddRefs(mListener),
                               PR_FALSE,
                               nsnull
                              );
  if (rv != NS_OK)
  {
    ErrorDialog(NS_LITERAL_STRING("errorStartingLoad"));
    return rv;
  }

#ifdef DEBUG_MODE
  printf("Calling AsyncOpen()...\n");
#endif
  rv = mChan->AsyncOpen(this, context);
  if (rv != NS_OK)
  {
    ErrorDialog(NS_LITERAL_STRING("errorOpeningChannel"));
    return rv;
  }

#ifdef DEBUG_MODE
  printf("Adding the observer...\n");
#endif

  // We hold a reference to ourselves while loading...
  AddRef();

  mDoc->AddObserver(NS_STATIC_CAST(nsIDocumentObserver*, this));
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLChannel::OnDataAvailable(
                                  nsIRequest *aRequest,
                                  nsISupports *aContext, 
                                  nsIInputStream *aInputStream,
                                  PRUint32 aOffset,
                                  PRUint32 aCount
                                 )
{
  return mListener->OnDataAvailable(this, aContext, aInputStream, aOffset, aCount);
}

NS_IMETHODIMP
mozCellMLChannel::OnStartRequest(
                                 nsIRequest *aRequest,
                                 nsISupports *aContext
                                )
{
  return mListener->OnStartRequest(this, aContext);
}

NS_IMETHODIMP
mozCellMLChannel::OnStopRequest(
                                nsIRequest *aRequest,
                                nsISupports *aContext,
                                nsresult aStatusCode
                               )
{
  if (aStatusCode != NS_OK)
  {
    ErrorDialog(NS_LITERAL_STRING("documentLoadFailed"));
    return aStatusCode;
  }
  return mListener->OnStopRequest(this, aContext, aStatusCode);
}

NS_IMPL_NSIDOCUMENTOBSERVER_CORE_STUB(mozCellMLChannel)
NS_IMPL_NSIDOCUMENTOBSERVER_REFLOW_STUB(mozCellMLChannel)
NS_IMPL_NSIDOCUMENTOBSERVER_STATE_STUB(mozCellMLChannel)
NS_IMPL_NSIDOCUMENTOBSERVER_CONTENT(mozCellMLChannel)
NS_IMPL_NSIDOCUMENTOBSERVER_STYLE_STUB(mozCellMLChannel)

void
mozCellMLChannel::BeginLoad(nsIDocument* aDocument)
{
}

void
mozCellMLChannel::EndLoad(nsIDocument* aDocument)
{
#ifdef DEBUG_MODE
  printf("EndLoad called...\n");
#endif
  nsresult rv;
  nsCOMPtr<nsIDOMDocument> dd = do_QueryInterface(aDocument);

  nsCOMPtr<nsIDOMElement> de;
  dd->GetDocumentElement(getter_AddRefs(de));
  if (de == nsnull)
  {
    loadCancelled = PR_TRUE;
    ErrorDialog(NS_LITERAL_STRING("noDocumentElement"));
    return;
  }
  nsAutoString de_name;
  de->GetLocalName(de_name);
  if (de_name.Equals(NS_LITERAL_STRING("parsererror")))
  {
    nsCOMPtr<nsIDOMNodeList> dnl;
    de->GetChildNodes(getter_AddRefs(dnl));
    nsCOMPtr<nsIArray> subText;
    FilterNodeListByType(dnl, getter_AddRefs(subText), nsIDOMNode::TEXT_NODE);
    nsCOMPtr<nsISimpleEnumerator> subTextE;
    subText->Enumerate(getter_AddRefs(subTextE));
    nsCOMPtr<nsISupports> txtNodeSupports;
    subTextE->GetNext(getter_AddRefs(txtNodeSupports));
    nsCOMPtr<nsIDOMText> txt(do_QueryInterface(txtNodeSupports));
    nsAutoString valueStr;
    nsCAutoString valueCStr;
    txt->GetData(valueStr);
    valueStr.Trim(" \t\r\n\f");    
    CopyUTF16toUTF8(valueStr, valueCStr);

    loadCancelled = PR_TRUE;
    ErrorDialog(NS_LITERAL_STRING("documentParseError"), valueCStr);
    return;
  }
  if (!de_name.Equals(NS_LITERAL_STRING("model")))
  {
    loadCancelled = PR_TRUE;
    ErrorDialog(NS_LITERAL_STRING("rootNotModel"));
    return;
  }

  nsCOMPtr<nsIURI> uri;
  nsCAutoString cnameStr, tmpStr;
  const char* rawNameStr, * p;
  mChan->GetOriginalURI(getter_AddRefs(uri));
  uri->GetSpec(cnameStr);
  
  rawNameStr = cnameStr.get();
  p = strrchr(rawNameStr, '/');
  if (p == nsnull)
    tmpStr = cnameStr;
  else
    tmpStr = p + 1;
  PRInt32 dotIndex = tmpStr.FindChar('.');
  if (dotIndex != -1)
    tmpStr.Cut(dotIndex, tmpStr.Length() - dotIndex);

  model = new mozCellML(tmpStr, isvalidator);

  rv = model->SetDOM(de);
  if (rv != NS_OK)
    return;
#ifdef DEBUG_MODE
  printf("Fetching imports from the model...\n");
#endif

  rv = model->FetchImports(getter_AddRefs(mImportRequests));
  if (rv != NS_OK)
  {
    loadCancelled = PR_TRUE;
    ErrorDialog(NS_LITERAL_STRING("errorGettingImportList"));
    return;
  }
#ifdef DEBUG_MODE
  printf("Counting imports...\n");
#endif
  mImportRequests->GetLength(&nRemaining);
#ifdef DEBUG_MODE
  printf("There are %u imports remaining to load...\n", nRemaining);
#endif

  if (nRemaining == 0 && !loadCancelled)
  {
    nsCOMPtr<mozICellMLUIService> us = do_GetService("@physiome.org/mozCellML/UIService;1");
    us->DisplayModel(model);
    Release();
    return;
  }

  nsCOMPtr<nsISimpleEnumerator> importE;
  mImportRequests->Enumerate(getter_AddRefs(importE));
  PRBool moreEls;
  while (importE->HasMoreElements(&moreEls), moreEls)
  {
    nsCOMPtr<nsISupports> importSupport;
    importE->GetNext(getter_AddRefs(importSupport));
    nsISupports *iS = importSupport;
    mozCellMLImportRequest* mcir =
      NS_STATIC_CAST(mozCellMLImportRequest*, NS_STATIC_CAST(nsIChannel*, iS));
    mcir->StartLoad(mLoadGroup, this);
  }

  Release();
}

void
mozCellMLChannel::ImportLoaded(nsIDOMElement* import,
                               nsIDOMElement* imported)
{
#ifdef DEBUG_MODE
  printf("mozCellMLChannel: Import loaded.\n");
#endif
  nRemaining--;
  nsresult rv = model->ImportLoaded(import, imported);
  if (rv != NS_OK)
  {
    loadCancelled = PR_TRUE;
    ErrorDialog(model->GetEarlyError());
    return;
  }

  if (nRemaining == 0)
  {
    nsresult rv = model->FetchImports(getter_AddRefs(mImportRequests));
    if (rv != NS_OK)
    {
      loadCancelled = PR_TRUE;
      ErrorDialog(NS_LITERAL_STRING("errorGettingImportList"));
      return;
    }
#ifdef DEBUG_MODE
    printf("Counting imports...\n");
#endif
    mImportRequests->GetLength(&nRemaining);
#ifdef DEBUG_MODE
    printf("There are %u imports remaining to load...\n", nRemaining);
#endif

    if (nRemaining == 0 && !loadCancelled)
    {
      nsCOMPtr<mozICellMLUIService> us = do_GetService("@physiome.org/mozCellML/UIService;1");
      us->DisplayModel(model);
      return;
    }
    
    nsCOMPtr<nsISimpleEnumerator> importE;
    mImportRequests->Enumerate(getter_AddRefs(importE));
    PRBool moreEls;
    while (importE->HasMoreElements(&moreEls), moreEls)
    {
      nsCOMPtr<nsISupports> importSupport;
      importE->GetNext(getter_AddRefs(importSupport));
      nsISupports *iS = importSupport;
      mozCellMLImportRequest* mcir =
        NS_STATIC_CAST(mozCellMLImportRequest*, NS_STATIC_CAST(nsIChannel*, iS));
      mcir->StartLoad(mLoadGroup, this);
    }
  }
}

void
mozCellMLChannel::ErrorDialog(mozCellMLLoadError* ierr)
{
  nsCOMPtr<mozCellMLLoadError> err = ierr;
  nsCOMPtr<nsIWindowWatcher> ww = do_GetService(NS_WINDOWWATCHER_CONTRACTID);
  nsCOMPtr<nsIDOMWindow> window;
  ww->OpenWindow(NULL, "chrome://mozcellml/content/loaderror.xul",
                 NULL, "chrome", err, getter_AddRefs(window));
}

void
mozCellMLChannel::ErrorDialog(const nsAString& msg)
{
  nsCOMPtr<nsIWindowWatcher> ww = do_GetService(NS_WINDOWWATCHER_CONTRACTID);
  nsCOMPtr<nsIDOMWindow> window;
  nsCOMPtr<mozCellMLLoadError> err = new mozCellMLLoadError(msg);
  ww->OpenWindow(NULL, "chrome://mozcellml/content/loaderror.xul",
                 NULL, "chrome", err, getter_AddRefs(window));
}

void
mozCellMLChannel::ErrorDialog(const nsAString& msg, const nsACString& arg1)
{
  nsCOMPtr<nsIWindowWatcher> ww = do_GetService(NS_WINDOWWATCHER_CONTRACTID);
  nsCOMPtr<nsIDOMWindow> window;
  nsCOMPtr<mozCellMLLoadError> err = new mozCellMLLoadError(msg, arg1);
  ww->OpenWindow(NULL, "chrome://mozcellml/content/loaderror.xul",
                 NULL, "chrome,dialog=no", err, getter_AddRefs(window));
}

void
mozCellMLChannel::ErrorDialog(const nsAString& msg, const nsACString& arg1, const nsACString& arg2)
{
  nsCOMPtr<nsIWindowWatcher> ww = do_GetService(NS_WINDOWWATCHER_CONTRACTID);
  nsCOMPtr<nsIDOMWindow> window;
  nsCOMPtr<mozCellMLLoadError> err = new mozCellMLLoadError(msg, arg1, arg2);
  ww->OpenWindow(NULL, "chrome://mozcellml/content/loaderror.xul",
                 NULL, "chrome,dialog=no", err, getter_AddRefs(window));
}

mozCellMLImportRequest::mozCellMLImportRequest(nsIDOMElement* iimportNode)
  : importNode(iimportNode)
{
}

mozCellMLImportRequest::~mozCellMLImportRequest()
{
  if (mDoc != nsnull)
  {
    mDoc->RemoveObserver(NS_STATIC_CAST(nsIDocumentObserver*, this));
    mDoc = nsnull;
  }
}

void
mozCellMLImportRequest::GetImportNode(nsIDOMElement** request)
{
  *request = importNode;
  NS_ADDREF(*request);
}

nsresult
mozCellMLImportRequest::StartLoad(
                                  nsILoadGroup* aLoadGroup,
                                  mozCellMLChannel* isuperChannel
                                 )
{
  nsresult rv;

  // Get the URL we are supposed to load...
  nsAutoString importURL;
#ifdef DEBUG_MODE
  printf("Fetching load URL...\n");
#endif
  nsCOMPtr<nsIURI> baseURL;
  isuperChannel->GetURI(getter_AddRefs(baseURL));
  importNode->GetAttributeNS(
                             NS_LITERAL_STRING("http://www.w3.org/1999/xlink"),
                             NS_LITERAL_STRING("href"),
                             importURL
                            );

  nsCAutoString cimportURL;
  CopyUTF16toUTF8(importURL, cimportURL);
#ifdef DEBUG_MODE
  printf("Load URL = %s\n", cimportURL.get());
#endif

  nsCOMPtr<nsIURI> impURI;
  rv = NS_NewURI(getter_AddRefs(impURI), cimportURL,
                 "utf8", baseURL);
  if (rv != NS_OK)
  {
    mozCellMLChannel::ErrorDialog(NS_LITERAL_STRING("importURLInvalid"),
                                  cimportURL);
    return rv;
  }

  impURI->GetSpec(cimportURL);
  CopyUTF8toUTF16(cimportURL, importURL);
  importNode->SetAttributeNS(
                             NS_LITERAL_STRING("http://www.w3.org/1999/xlink"),
                             NS_LITERAL_STRING("href"),
                             importURL
                            );

  rv = NS_NewChannel(getter_AddRefs(mChannel), impURI);
  if (rv != NS_OK)
  {
    mozCellMLChannel::ErrorDialog(NS_LITERAL_STRING("importChannelError"),
                                  cimportURL);
    return rv;
  }

  superChannel = isuperChannel;
  static NS_DEFINE_CID(
                       kXMLDocumentCID,
                       NS_XMLDOCUMENT_CID
                      );
  mDoc = do_CreateInstance("@mozilla.org/xml/xml-document;1");
  if (mDoc == nsnull)
    mDoc = do_CreateInstance(kXMLDocumentCID);
  if (mDoc == nsnull)
  {
#ifdef DEBUG_MODE
    printf("Failing because we can't make an XML Document.\n");
#endif
    return NS_ERROR_OUT_OF_MEMORY;
  }

  nsCOMPtr<nsIStreamListener> aDocListener;

#ifdef DEBUG_MODE
  printf("Starting import document load...\n");
#endif
  rv = mDoc->StartDocumentLoad(
                               "loadAsData",
                               this,
                               aLoadGroup,
                               nsnull,
                               getter_AddRefs(mListener),
                               PR_FALSE,
                               nsnull
                              );
  if (rv != NS_OK)
  {
    mozCellMLChannel::ErrorDialog(NS_LITERAL_STRING("importLoadError"),
                                  cimportURL);
    return rv;
  }

  NS_ENSURE_SUCCESS(rv, rv);
#ifdef DEBUG_MODE
  printf("Calling AsyncOpen() for import...\n");
#endif
  rv = mChannel->AsyncOpen(this, nsnull);
  if (rv != NS_OK)
  {
    mozCellMLChannel::ErrorDialog(NS_LITERAL_STRING("importLoadError"),
                                  cimportURL);
    return rv;
  }

#ifdef DEBUG_MODE
  printf("Adding observer...\n");
#endif
  mDoc->AddObserver(NS_STATIC_CAST(nsIDocumentObserver*, this));
  return NS_OK;
}

NS_IMPL_NSIDOCUMENTOBSERVER_CORE_STUB(mozCellMLImportRequest)
NS_IMPL_NSIDOCUMENTOBSERVER_REFLOW_STUB(mozCellMLImportRequest)
NS_IMPL_NSIDOCUMENTOBSERVER_STATE_STUB(mozCellMLImportRequest)
NS_IMPL_NSIDOCUMENTOBSERVER_CONTENT(mozCellMLImportRequest)
NS_IMPL_NSIDOCUMENTOBSERVER_STYLE_STUB(mozCellMLImportRequest)

NS_IMETHODIMP
mozCellMLImportRequest::GetName(nsACString& name)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->GetName(name);
}

NS_IMETHODIMP
mozCellMLImportRequest::IsPending(PRBool* pend)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->IsPending(pend);
}

NS_IMETHODIMP
mozCellMLImportRequest::GetStatus(nsresult* stat)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->GetStatus(stat);
}

NS_IMETHODIMP
mozCellMLImportRequest::Cancel(nsresult ret)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->Cancel(ret);
}

NS_IMETHODIMP
mozCellMLImportRequest::Suspend()
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->Suspend();
}

NS_IMETHODIMP
mozCellMLImportRequest::Resume()
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->Resume();
}

NS_IMETHODIMP
mozCellMLImportRequest::SetLoadGroup(nsILoadGroup* aLoadGroup)
{
  NS_ASSERTION(doc == nsnull, "SetLoadGroup called after AsyncOpen()");
  mLoadGroup = aLoadGroup;
  if (mChannel != nsnull)
    return mChannel->SetLoadGroup(aLoadGroup);
  else
    return NS_OK;
}

NS_IMETHODIMP
mozCellMLImportRequest::GetLoadGroup(nsILoadGroup** aLoadGroup)
{
#ifdef DEBUG_MODE
  printf("GetLoadGroup called...\n");
#endif
  if (mLoadGroup != nsnull)
  {
#ifdef DEBUG_MODE
    printf("Returning the LoadGroup stored in mLoadGroup...\n");
#endif
    *aLoadGroup = mLoadGroup;
    NS_ADDREF(*aLoadGroup);
    return NS_OK;
  }
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->GetLoadGroup(aLoadGroup);
}

NS_IMETHODIMP
mozCellMLImportRequest::SetLoadFlags(nsLoadFlags loadFlags)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->SetLoadFlags(loadFlags);
}

NS_IMETHODIMP
mozCellMLImportRequest::GetLoadFlags(nsLoadFlags* loadFlags)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->GetLoadFlags(loadFlags);
}

NS_IMETHODIMP
mozCellMLImportRequest::GetOriginalURI(nsIURI** aURI)
{
  if (mURI == nsnull)
  {
    // This is actually only for the synchronous mode(otherwise we use StartLoad)
    nsAutoString importURL;
    importNode->GetAttributeNS(NS_LITERAL_STRING("http://www.w3.org/1999/xlink"),
                               NS_LITERAL_STRING("href"),
                               importURL);
    
    nsCAutoString cimportURL;
    CopyUTF16toUTF8(importURL, cimportURL);
#ifdef DEBUG_MODE
    printf("mcir: Load URL = %s\n", cimportURL.get());
#endif

    nsresult rv = NS_NewURI(aURI, cimportURL);
    NS_ENSURE_SUCCESS(rv, rv);
    mURI = *aURI;
    return NS_OK;
  }
  *aURI = mURI;
  NS_ADDREF(*aURI);
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLImportRequest::SetOriginalURI(nsIURI* aURI)
{
  mURI = aURI;
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLImportRequest::GetURI(nsIURI** aURI)
{
  if (mChannel == nsnull)
  {

    return GetOriginalURI(aURI);
  }
  return mChannel->GetURI(aURI);
}

NS_IMETHODIMP
mozCellMLImportRequest::GetOwner(nsISupports** owner)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->GetOwner(owner);
}

NS_IMETHODIMP
mozCellMLImportRequest::SetOwner(nsISupports* owner)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->SetOwner(owner);
}

NS_IMETHODIMP
mozCellMLImportRequest::SetNotificationCallbacks(nsIInterfaceRequestor* notcb)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->SetNotificationCallbacks(notcb);
}

NS_IMETHODIMP
mozCellMLImportRequest::GetNotificationCallbacks(nsIInterfaceRequestor** notcb)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->GetNotificationCallbacks(notcb);
}

NS_IMETHODIMP
mozCellMLImportRequest::GetSecurityInfo(nsISupports** secInfo)
{
  if (mChannel == nsnull)
  {
    *secInfo = nsnull;
    return NS_OK;
  }
  return mChannel->GetSecurityInfo(secInfo);
}

NS_IMETHODIMP
mozCellMLImportRequest::GetContentType(nsACString& ctype)
{
#ifdef DEBUG_MODE
  printf("GetContentType called.\n");
#endif
  ctype.Assign("text/xml");
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLImportRequest::SetContentType(const nsACString& ctype)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->SetContentType(ctype);
}

NS_IMETHODIMP
mozCellMLImportRequest::GetContentCharset(nsACString& ccharset)
{
  if (mChannel == nsnull)
  {
    ccharset.Assign(NS_LITERAL_CSTRING("utf8"));
    return NS_OK;
  }
  return mChannel->GetContentCharset(ccharset);
}

NS_IMETHODIMP
mozCellMLImportRequest::SetContentCharset(const nsACString& ccharset)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->SetContentCharset(ccharset);
}

NS_IMETHODIMP
mozCellMLImportRequest::GetContentLength(PRInt32* length)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->GetContentLength(length);
}

NS_IMETHODIMP
mozCellMLImportRequest::SetContentLength(PRInt32 length)
{
  if (mChannel == nsnull)
    return NS_ERROR_FAILURE;
  return mChannel->SetContentLength(length);
}

void
mozCellMLImportRequest::BeginLoad(nsIDocument* aDocument)
{
}

void
mozCellMLImportRequest::EndLoad(nsIDocument* aDocument)
{
#ifdef DEBUG_MODE
  printf("Import load complete.\n");
#endif
  nsCOMPtr<nsIDOMDocument> dd = do_QueryInterface(aDocument);

  nsCAutoString cimportURL;
  if (mURI != nsnull)
    mURI->GetSpec(cimportURL);

  nsCOMPtr<nsIDOMElement> de;
  dd->GetDocumentElement(getter_AddRefs(de));
  if (de == nsnull)
  {
    mozCellMLChannel::ErrorDialog(NS_LITERAL_STRING("importNoDocumentElement"),
                                  cimportURL);
    return;
  }
  nsAutoString de_name;
  de->GetLocalName(de_name);
  if (de_name.Equals(NS_LITERAL_STRING("parsererror")))
  {
    nsCOMPtr<nsIDOMNodeList> dnl;
    de->GetChildNodes(getter_AddRefs(dnl));
    nsCOMPtr<nsIArray> subText;
    FilterNodeListByType(dnl, getter_AddRefs(subText), nsIDOMNode::TEXT_NODE);
    nsCOMPtr<nsISimpleEnumerator> subTextE;
    subText->Enumerate(getter_AddRefs(subTextE));
    nsCOMPtr<nsISupports> txtNodeSupports;
    subTextE->GetNext(getter_AddRefs(txtNodeSupports));
    nsCOMPtr<nsIDOMText> txt(do_QueryInterface(txtNodeSupports));
    nsAutoString valueStr;
    nsCAutoString valueCStr;
    txt->GetData(valueStr);
    valueStr.Trim(" \t\r\n\f");    
    CopyUTF16toUTF8(valueStr, valueCStr);

    mozCellMLChannel::ErrorDialog(
                                  NS_LITERAL_STRING("importDocumentParseError"),
                                  cimportURL, valueCStr);
    return;
  }
  if (!de_name.Equals(NS_LITERAL_STRING("model")))
  {
    mozCellMLChannel::ErrorDialog(
                                  NS_LITERAL_STRING("importRootNotModel"),
                                  cimportURL);
    return;
  }

  superChannel->ImportLoaded(importNode, de);

  //  Break cycle...
  superChannel = nsnull;
}

NS_IMETHODIMP
mozCellMLImportRequest::OnDataAvailable(
                                        nsIRequest *aRequest,
                                        nsISupports *aContext, 
                                        nsIInputStream *aInputStream,
                                        PRUint32 aOffset,
                                        PRUint32 aCount
                                       )
{
  if (mListener != nsnull)
    return mListener->OnDataAvailable(this, aContext, aInputStream, aOffset, aCount);
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLImportRequest::OnStartRequest(
                                 nsIRequest *aRequest,
                                 nsISupports *aContext
                                      )
{
  if (mListener != nsnull)
    return mListener->OnStartRequest(this, aContext);
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLImportRequest::OnStopRequest(
                                      nsIRequest *aRequest,
                                      nsISupports *aContext,
                                      nsresult aStatusCode
                                     )
{
  if (aStatusCode != NS_OK)
  {
    nsCAutoString cimportURL;
    if (mURI != nsnull)
      mURI->GetSpec(cimportURL);
    mozCellMLChannel::ErrorDialog(NS_LITERAL_STRING("importLoadFailed"),
                                  cimportURL);
    return aStatusCode;
  }
  if (mListener != nsnull)
    return mListener->OnStopRequest(this, aContext, aStatusCode);
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLImportRequest::Open(nsIInputStream** iis)
{
#ifdef DEBUG_MODE
  printf("Failing because we can't Open()...\n");
#endif
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
mozCellMLImportRequest::AsyncOpen(
                                  nsIStreamListener* aListener,
                                  nsISupports* context
                                 )
{
  mListener = aListener;
  if (mURI == nsnull)
    GetOriginalURI(getter_AddRefs(mURI));

  nsresult rv = NS_NewChannel(getter_AddRefs(mChannel), mURI);
  NS_ENSURE_SUCCESS(rv, rv);

  rv = mChannel->AsyncOpen(this, nsnull);
  NS_ENSURE_SUCCESS(rv, rv);

  return NS_OK;
}
