/**
 * Return a string identifying the focused control, suitable to use to restore
 * focus later...
 * @param w The VWindow to consider.
 * @return A string representing the focus.
 */
function IPGetFocusedControl(w)
{
  var tab =
    {
      startx: w.startXtext,
      stopx: w.stopXtext,
      gridx: w.gridXtext,
      relErr: w.relErrtext,
      absErr: w.absErrtext,
      maxStep: w.maxSteptext
    };
  var allR = "";
  for (name in tab)
  {
    allR = allR + "," + tab[name].selectionStart + "-" + tab[name].selectionEnd;
  }

  for (name in tab)
  {
    if (tab[name].isfocused)
      return name + allR;
  }
  return "nofocus" + allR;
}

/**
 * Sets the focus based on a string from GetFocusedControl.
 * @param w The VWindow to consider.
 * @param f The focus string returned by GetFocusedControl.
 */
function IPSetFocusedControl(w, f)
{
  var allf = f.split(',');

  var sel = w.win.getSelection();
  sel.removeAllRanges();

  var tab =
    {
      startx: w.startXtext,
      stopx: w.stopXtext,
      gridx: w.gridXtext,
      relErr: w.relErrtext,
      absErr: w.absErrtext,
      maxStep: w.maxSteptext
    };

  var i = 0;
  for (name in tab)
  {
    i++;
    var p = allf[i].split('-');
    tab[name].selectionStart = p[0];
    tab[name].selectionEnd = p[1];
  }

  if (tab[allf[0]] == null)
    return;

  tab[allf[0]].isfocused = true;
  var x = tab[allf[0]];
  setTimeout(function() {x.focus();}, 1);
}

/**
 * Initialise the integration parameters general window view.
 * @param content The content area.
 */
function InitIntParams(content)
{
  var stk = content.getElementsByTagName("deck").item(0);
  var vbc = stk.getElementsByTagName("vbox");
  var mainbox = vbc.item(0);
  var errorbox = vbc.item(1);
  var hbc = mainbox.getElementsByTagName("hbox");
  var hb = hbc.item(0);
  var lc = hb.getElementsByTagName("label");
  this.beginBoundRange = lc.item(0);
  this.endBoundRange = lc.item(2);
  var tc = hb.getElementsByTagNameNS("http://www.w3.org/1999/xhtml", "input");
  this.startXtext = tc.item(0);
  this.startXtext.window = this;
  this.stopXtext = tc.item(1);
  this.stopXtext.window = this;
  hb = hbc.item(1);
  this.gridXtext = hb.getElementsByTagNameNS("http://www.w3.org/1999/xhtml",
                                           "input").item(0);
  this.gridXtext.window = this;
  this.gridXunits = hb.getElementsByTagName("label").item(0);
  this.relErrtext = hbc.item(2).getElementsByTagNameNS
    ("http://www.w3.org/1999/xhtml", "input").item(0);
  this.relErrtext.window = this;
  this.absErrtext = hbc.item(3).getElementsByTagNameNS
    ("http://www.w3.org/1999/xhtml", "input").item(0);
  this.absErrtext.window = this;
  this.maxSteptext = hbc.item(4).getElementsByTagNameNS
    ("http://www.w3.org/1999/xhtml", "input").item(0);
  this.maxSteptext.window = this;
  this.stack = stk;
  this.errorText = errorbox.getElementsByTagName("label").item(0);
  this.errorText.setAttribute("value", "No model currently loaded.");
  this.stack.setAttribute("selectedIndex", 1);
  this.actionStack = content.getElementsByTagName("deck").item(1);
  var bl = this.actionStack.getElementsByTagName("button");
  this.startButton = bl.item(0);
  this.cancelButton = bl.item(1);
  this.startButton.window = this;
  this.cancelButton.window = this;

}

/**
 * Called when the integration parameter view is going away.
 */
function FinishIntParams(destroy)
{
  if (this.activeModel != null)
  {
    var fc = IPGetFocusedControl(this);
    this.activeModel.xmlmi.paramFocusWhere = fc;
  }
}

/**
 * Called whenever the view needs to update the model.
 * @param m The new ModelInstance.
 */
function IntParamsUpdateModel(m)
{
  this.integrationparams_Finish(true);

  this.activeModel = m;

  var allvars = new Object();
  var numvars = new Object();
  m.cellml.getAllVariables(Components.interfaces.mozICellML.NO_IN_VARIABLES,
                           allvars, numvars);

  numvars = numvars.value;
  var cellmlvar, i;
  var nbound = 0;
  var bound;
  for (i = 0; i < numvars; i++)
  {
    cellmlvar = allvars.value[i];
    if (cellmlvar.type == Components.interfaces.mozICellMLVariable.BOUND)
    {
      nbound++;
      bound = cellmlvar;
    }
  }
  if (nbound == 0 || nbound == 2)
  {
    this.errorText.
      setAttribute("value",
                   "mozCellML can only integrate models with exactly one " +
                   "bound variable, not " + nbound);
    this.stack.setAttribute("selectedIndex", 1);
    return;
  }
  
  var un = bound.units;
  var n = bound.name;

  this.beginBoundRange.setAttribute("value", n + ": (");
  this.endBoundRange.setAttribute("value", ") " + un);

  this.stack.setAttribute("selectedIndex", 0);

  this.startXtext.value = m.startx;
  this.stopXtext.value = m.stopx;
  this.gridXtext.value = m.gridx;
  this.relErrtext.value = m.relError;
  this.absErrtext.value = m.absError;
  this.maxSteptext.value = m.maxStep;
  this.actionStack.setAttribute("selectedIndex", m.integrationState);

  if (this.activeModel.xmlmi.paramFocusWhere != null)
    IPSetFocusedControl(this, this.activeModel.xmlmi.paramFocusWhere);
}

function DoCompileIntegrator(sb, vwin)
{
  // Get a temporary file name, to store the C code in...
  var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                             .getService(Components
                                         .interfaces
                                         .nsIDirectoryServiceProvider);
  var p = new Object();
  var lf = dirService.getFile("TmpD", p)
                     .QueryInterface(Components.interfaces.nsILocalFile);
  lf.appendRelativePath("integrator.c");
  lf.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
                  /*0600*/384);
  var cfile = lf.path;
  var lfx = dirService.getFile("TmpD", p)
                      .QueryInterface(Components.interfaces.nsILocalFile);
  if (navigator.platform.substring(0, 5) == "Win32")
    lfx.appendRelativePath("integrator.exe");
  else
    lfx.appendRelativePath("integrator");

  lfx.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
                  /*0700*/448);
  var xfile = lfx.path;

  var cc = GetCompiler();

  // XXX not all compilers take the same arguments...
  cc += " " + cfile + " -o " + xfile;

  sb.label = vwin.strb.getString("compilingIntegrator");
  try
  {
    vwin.activeModel.xmlmi.integrator =
      vwin.activeModel.cellml.newIntegrator(cfile, cc, xfile);
    if (vwin.activeModel.xmlmi.integrator == null)
      alert("Integrator compile failed!");
    sb.setAttribute("label", vwin.strb.getString("integratorCompiled"));
  }
  catch (e)
  {
    var valerr = vwin.activeModel.cellml.checkValidity();
    if (valerr == "")
      sb.setAttribute("label", vwin.strb.getString("compilationFailed"));
    else
    {
      sb.setAttribute("label", vwin.strb.getFormattedString("integrationFailedReason",
                                                [valerr]));
    }
  }
  lf.remove(false);
}

/**
 * Called when the user clicks the "Integrate Model" button.
 */
function StartIntegration(button)
{
  var w = button.window;

  if (w.activeModel == null)
    return;

  var m = w.activeModel;

  if (m.integration_started)
    return;

  var sb = GetStatusBar();

  if (isNaN(m.startx) || !isFinite(m.startx))
  {
    sb.label = w.strb.getString("invalidStartingBound");
    return;
  }
  if (isNaN(m.stopx) || !isFinite(m.stopx))
  {
    sb.label = w.strb.getString("invalidEndingBound");
    return;
  }
  if (isNaN(m.gridx) || !isFinite(m.gridx))
  {
    sb.label = w.strb.getString("invalidBoundResolution");
    return;
  }
  if (isNaN(m.relError) || !isFinite(m.relError))
  {
    sb.label = w.strb.getString("invalidRelErr");
    return;
  }
  if (isNaN(m.absError) || !isFinite(m.absError))
  {
    sb.label = w.strb.getString("invalidAbsErr");
    return;
  }
  if (isNaN(m.maxStep) || !isFinite(m.maxStep))
  {
    sb.label = strb.getString("invalidMaxStep");
    return;
  }
  var diff = m.stopx - m.startx;
  if (diff < 0)
    diff = -diff;

  // Now that we have valid values, the model is no longer dirty.
  m.dirty = false;
  // The user can only integrate each ModelInstance once in the new UI.
  m.integration_started = true;
  w.actionStack.setAttribute("selectedIndex", "1");
  m.integrationState = 1;

  // Step 1 is to get hold of a blank integration task...
  if (m.integrationtask == null)
  {
    // We need an integrator before we can create tasks...
    if (m.xmlmi.integrator == null)
    {
      DoCompileIntegrator(sb, w);
    }
    m.integrationtask = m.xmlmi.integrator.newTask();
  }

  // We also need a cmgui. Unlike integrators, cmguis are per-ModelInstance not
  // per-XMLModelInstance.
  if (m.cmgui == null)
  {
    DoLaunchCmgui(sb, w);
  }
  else // this gets called automatically if DoLaunchCmgui works.
    DoStartIntegration(w, m);
}

function CmguiLauncher(w, m)
{
  this.vwin = w;
  this.model = m;
  return this;
}

CmguiLauncher.prototype =
{
  QueryInterface: function(aIID)
  {
    if (
        aIID.equals(Components.interfaces.CmguiIApplicationBootstrap) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },

  cmguiReady: function(cdf)
  {
    GetStatusBar().label = "Cmgui has started.";
    try
    {
      this.model.cmgui = cdf.createCommandData();
    }
    catch (e)
    {
      alert("Failure creating command data.");
    }

    /* We now do as much setup as possible prior to creating any windows. This
     * will greatly improve reliability and speed on Win32, as it avoids the
     * need to process windows messages to avoid deadlock.
     */
    DoCmguiOneOffSetup(this.model);

    DoStartIntegration(this.vwin, this.model);
    this.vwin = null;
    this.model = null;
  }
};

function DoLaunchCmgui(sb, w)
{
  sb.label = "Launching cmgui";
  var clp = ProxyToMain(new CmguiLauncher(w, w.activeModel),
                        Components.interfaces.CmguiIApplicationBootstrap);
  w.activeModel.cellml.startCmgui(clp);
}

function IntegrationDoneObserver(w, m)
{
  this.w = w;
  this.m = m;
  return this;
}

IntegrationDoneObserver.prototype =
{
  scaleChanged: function(v, ns)
  {
    this.m.scales[v.importNumber + ":" + v.componentName + ":" + v.name] = ns;
  },

  progress: function(lastBound)
  {
    this.m.lastBound = lastBound;
    UpdateAvailableBound(this.m);
  },

  done: function()
  {
    this.w.actionStack.setAttribute("selectedIndex", "2");
    this.m.integrationState = 2;
  },

  halted: function()
  {
    this.w.actionStack.setAttribute("selectedIndex", "0");
    this.m.integrationState = 0;
    this.m.integration_started = false;
    this.m.integrationtask = null;
    GetStatusBar.setAttribute("label", "Integration halted");
  },

  failed: function(msg)
  {
    this.w.actionStack.setAttribute("selectedIndex", "0");
    this.m.integrationState = 0;
    this.m.integration_started = false;
    this.m.integrationtask = null;
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = GetStatusBar();
    var idx = msg.indexOf('/');
    if (idx == -1)
    {
      var err = strb.getFormattedString("integrationFailedReason", [strb.getString(msg)]);
      statusBar.setAttribute("label", err);
    }
    else
    {
      var emsg1 = msg.substring(0, idx);
      var emsg2 = msg.substring(idx + 1);
      var err = strb.getFormattedString("integrationFailedReason", [strb.getFormattedString(emsg1, [emsg2])]);
      statusBar.setAttribute("label", err);
    }
  }
};

function DoStartIntegration(w, m)
{
  if (m.integrationtask == null)
    return;
  var t = m.integrationtask;

  // Configure points to stream to cmgui as they load(this also keeps a local
  // copy allowing raw results to be retrieved).
  m.scales = {};
  m.gsm = new GraphScaleMonitor(m);
  m.gsm.addMonitor(new IntegrationDoneObserver(w, m));

  m.results = t.sendPointsToZinc(
                                 m.xmlmi.cellml,
                                 m.gsm,
                                 m.startx, m.stopx, m.gridx,
                                 m.xmlmi.ivariables, m.xmlmi.nivariables,
                                 m.cmgui.valuefield, m.xmlmi.nivariables,
                                 m.cmgui.cmgui, m.cmgui.timeSequencePackage,
                                 m.cmgui.pointFieldCreator,
                                 m.cmgui.dataNode
                                );

  var bv = m.xmlmi.bvar;
  m.scales[bv.importNumber + ":" + bv.componentName + ":" + bv.name] = Math.max(Math.abs(m.startx),Math.abs(m.stopx));
  t.start(m, m.startx, m.stopx, m.gridx, m.absError, m.relError, m.maxStep);
}

/**
 * Called when the user clicks the "Cancel Integration" button.
 */
function StopIntegration(button)
{
}

function ChangeIntegrationStart(tb, e)
{
  var w = tb.window;
  if (w.activeModel == null)
    return;

  if (!w.activeModel.dirty)
  {
    /* They must have used a non-key means of editing(e.g. clipboard). This
     * hack will probably get the selection wrong, but at least it should
     * get the text entered correct.
     */
    var x = tb.value;
    InitialConditionKey(tb, e);
    tb.value = x;
  }

  w.activeModel.startx = parseFloat(tb.value);
}

function ChangeIntegrationEnd(tb, e)
{
  var w = tb.window;
  if (w.activeModel == null)
    return;

  if (!w.activeModel.dirty)
  {
    /* They must have used a non-key means of editing(e.g. clipboard). This
     * hack will probably get the selection wrong, but at least it should
     * get the text entered correct.
     */
    var x = tb.value;
    InitialConditionKey(tb, e);
    tb.value = x;
  }

  w.activeModel.stopx = parseFloat(tb.value);
}

function ChangeIntegrationGrid(tb, e)
{
  var w = tb.window;
  if (w.activeModel == null)
    return;

  if (!w.activeModel.dirty)
  {
    /* They must have used a non-key means of editing(e.g. clipboard). This
     * hack will probably get the selection wrong, but at least it should
     * get the text entered correct.
     */
    var x = tb.value;
    InitialConditionKey(tb, e);
    tb.value = x;
  }

  w.activeModel.gridx = parseFloat(tb.value);
}

function ChangeIntegrationRelErr(tb, e)
{
  var w = tb.window;
  if (w.activeModel == null)
    return;

  if (!w.activeModel.dirty)
  {
    /* They must have used a non-key means of editing(e.g. clipboard). This
     * hack will probably get the selection wrong, but at least it should
     * get the text entered correct.
     */
    var x = tb.value;
    InitialConditionKey(tb, e);
    tb.value = x;
  }

  w.activeModel.relError = parseFloat(tb.value);
}

function ChangeIntegrationAbsErr(tb, e)
{
  var w = tb.window;
  if (w.activeModel == null)
    return;

  if (!w.activeModel.dirty)
  {
    /* They must have used a non-key means of editing(e.g. clipboard). This
     * hack will probably get the selection wrong, but at least it should
     * get the text entered correct.
     */
    var x = tb.value;
    InitialConditionKey(tb, e);
    tb.value = x;
  }

  w.activeModel.absError = parseFloat(tb.value);
}

function ChangeIntegrationMaxStep(tb, e)
{
  var w = tb.window;
  if (w.activeModel == null)
    return;

  if (!w.activeModel.dirty)
  {
    /* They must have used a non-key means of editing(e.g. clipboard). This
     * hack will probably get the selection wrong, but at least it should
     * get the text entered correct.
     */
    var x = tb.value;
    InitialConditionKey(tb, e);
    tb.value = x;
  }

  w.activeModel.maxStep = parseFloat(tb.value);
}

function IntParamsSaveContent(node)
{
  node.setAttribute("startx", this.startXtext.value);
  node.setAttribute("stopx", this.stopXtext.value);
  node.setAttribute("gridx", this.gridXtext.value);
  node.setAttribute("abserr", this.absErrtext.value);
  node.setAttribute("relerr", this.relErrtext.value);
  node.setAttribute("maxstep", this.maxSteptext.value);
}

function IntParamsRestoreContent(node)
{
  this.startXtext.value = node.getAttribute("startx");
  this.stopXtext.value = node.getAttribute("stopx");
  this.gridXtext.value = node.getAttribute("gridx");
  this.absErrtext.value = node.getAttribute("abserr");
  this.relErrtext.value = node.getAttribute("relerr");
  this.maxSteptext.value = node.getAttribute("maxstep");
}
