function SetupIntegrationPane()
{
  ParsePageQuery();
  window.model = GetCellMLModel();

  var numvars = new Object();
  var variables = new Object();
  window.model.getAllVariables(Components.interfaces.mozICellML.NO_INTERFACE_IN,
                               variables, numvars);
  window.variables = variables.value;
  numvars = numvars.value;
  var bc = 0;

  var i;
  for (i = 0; i < numvars; i++)
  {
    var v = window.variables[i];
    if (v.type == Components.interfaces.mozICellMLVariable.BOUND)
    {
      bc++;
      window.bound = v.name;
      window.boundunit = v.units;
      var boundname = document.getElementById("boundname");
      boundname.value = window.bound + " interval: (";
      var unitsep = document.getElementById("unitsep");
      unitsep.value = window.boundunit + " ,";
      var unitstop = document.getElementById("unitstop");
      unitstop.value = window.boundunit + ")";
      var unitgrid = document.getElementById("unitgrid");
      unitgrid.value = window.boundunit;
    }
  }
  if (bc != 1)
  {
    window.location.href = "nointegration.html";
  }
}

function CompileIntegrator()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var statusBar = GetStatusBar();

  if (window.boundcount == 0)
  {
    statusBar.label = strb.getString("mustHaveBoundVar");
  }
  else if (window.boundcount > 1)
  {
    statusBar.label = strb.getString("multipleBoundVars");
  }

  // Get a temporary file name, to store the C code in...
  var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                             .getService(Components
                                         .interfaces
                                         .nsIDirectoryServiceProvider);
  var p = new Object();
  var lf = dirService.getFile("TmpD", p)
                     .QueryInterface(Components.interfaces.nsILocalFile);
  lf.appendRelativePath("integrator.c");
  lf.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
                  /*0600*/384);
  var cfile = lf.path;
  var lfx = dirService.getFile("TmpD", p)
                      .QueryInterface(Components.interfaces.nsILocalFile);
  if (navigator.platform.substring(0, 5) == "Win32")
    lfx.appendRelativePath("integrator.exe");
  else
    lfx.appendRelativePath("integrator");

  lfx.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
                  /*0700*/448);
  var xfile = lfx.path;

  var cc = GetCompiler();

  // XXX not all compilers take the same arguments...
  cc += " " + cfile + " -o " + xfile;

  statusBar.label = strb.getString("compilingIntegrator");
  try
  {
    window.integrator = window.model.newIntegrator(cfile, cc, xfile);
    statusBar.label = strb.getString("integratorCompiled");
    ChangeIntegratorButtons(document, true, false);
  }
  catch (e)
  {
    var valerr = window.model.checkValidity();
    if (valerr == "")
      statusBar.label = strb.getString("compilationFailed");
    else
    {
      statusBar.label = strb.getFormattedString("integrationFailedReason",
                                                [valerr]);
    }
    ChangeIntegratorButtons(document, false, false);
  }
  lf.remove(false);
}

function CleanupIntegrationPane()
{
  // Lock the user out from doing bad things while closing...
  ChangeIntegratorButtons(document, false, false);
  if (window.integrator != null)
  {
    window.integrator.destroy();
    window.integrator = null;
  }
}

function IntegrationScaleWatcher(npoints)
{
  this.curpoint = 0;
  this.npoints = npoints + 1;
  this.launchDoc = document;
  return this;
}

IntegrationScaleWatcher.prototype =
{
  QueryInterface: function(aIID)
  {
    if (
        aIID.equals(Components.interfaces.mozCellMLIScaleWatcher) ||
        aIID.equals(Components.interfaces.nsISupports) ||
        aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
        aIID.equals(Components.interfaces.CmguiIApplicationBootstrap)
       )
    {
      return this;
    }
    throw Components.results.NS_NOINTERFACE;
  },

  cmguiReady: function(cdf)
  {
    this.commandData = cdf.createCommandData();
    var m = GetMyMenu();
    window.resultsoption =
      m.AddOption(
                  "Results", "integration_results_pane.xul?id=" +
                  window.query["id"] + "&page=" +
                  m.mainwin.nextid
                 );
    window.graphsoption =
      m.AddOption("Graphs", "integration_results_pane.xul?id=" +
                  window.query["id"] + "&page=" +
                  m.mainwin.nextid);
  },

  activateIntegration: function()
  {
    this.live = true;
    if (!this.closed)
    {
      SetupZincForNewIntegrator(this);
      ChangeIntegratorButtons(this.launchDoc, true, true);
      this.integrator.start
      (
        window.model, this.startx, this.stopx, this.gridx,
        this.abserr, this.relerr, this.maxstep
      );
      var statusBar = this.launchDoc.getElementById('status-bar-display');
      var strb = this.launchDoc.getElementById("mozcellml-string-bundle");
      statusBar.label = strb.getString("integrationStarted");
    }
  },

  scaleChanged: function(variable, newMaxAbs)
  {
    ComputeCmguiScale(this, variable, newMaxAbs);
  },

  progress: function(percent)
  {
    if (this.meter != null)
      this.meter.setAttribute("value", percent);
  },

  done: function()
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("integrationDone");
    if (window.integrator != null)
      ChangeIntegratorButtons(document, true, false);
    this.live = false;
    if (this.meter != null)
      this.meter.setAttribute("value", "100%");
  },

  halted: function()
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("integrationHalted");
    this.live = false;
    if (window.integrator != null)
      ChangeIntegratorButtons(document, true, false);
  },

  failed: function(emsg)
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    var idx = emsg.indexOf('/');
    if (idx == -1)
    {
      var err = strb.getFormattedString("integrationFailedReason", [strb.getString(emsg)]);
      statusBar.label = err;
      alert(err);
      this.live = false;
      if (window.integrator != null)
        ChangeIntegratorButtons(document, true, false);
    }
    else
    {
      var emsg1 = emsg.substring(0, idx);
      var emsg2 = emsg.substring(idx + 1);
      var err = strb.getFormattedString("integrationFailedReason", [strb.getFormattedString(emsg1, [emsg2])]);
      statusBar.label = err;
      alert(err);
      this.live = false;
      if (window.integrator != null)
        ChangeIntegratorButtons(document, true, false);
    }
  },

  live: false,
  closed: false
};

function StartIntegration()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var statusBar = document.getElementById('status-bar-display');

  /* Validate the results... */
  var startx = parseFloat(document.getElementById('startx').value);
  var stopx = parseFloat(document.getElementById('stopx').value);
  var gridx = parseFloat(document.getElementById('gridx').value);
  var relerr = parseFloat(document.getElementById('relerr').value);
  var abserr = parseFloat(document.getElementById('relerr').value);
  var maxstep = parseFloat(document.getElementById('maxstep').value);
  if (isNaN(startx) || !isFinite(startx))
  {
    statusBar.label = strb.getString("invalidStartingBound");
    return;
  }
  if (isNaN(stopx) || !isFinite(stopx))
  {
    statusBar.label = strb.getString("invalidEndingBound");
    return;
  }
  if (isNaN(gridx) || !isFinite(gridx))
  {
    statusBar.label = strb.getString("invalidBoundResolution");
    return;
  }
  if (isNaN(relerr) || !isFinite(relerr))
  {
    statusBar.label = strb.getString("invalidRelErr");
    return;
  }
  if (isNaN(abserr) || !isFinite(abserr))
  {
    statusBar.label = strb.getString("invalidAbsErr");
    return;
  }
  if (isNaN(maxstep) || !isFinite(maxstep))
  {
    statusBar.label = strb.getString("invalidMaxStep");
    return;
  }

  var diff = stopx - startx;
  if (diff < 0)
    diff = -diff;
  var irw = new IntegrationScaleWatcher(parseInt(diff / gridx));
  irw.startx = startx;
  irw.stopx = stopx;
  irw.gridx = gridx;
  irw.abserr = abserr;
  irw.relerr = relerr;
  irw.maxstep = maxstep;
  irw.model = window.model;
  irw.variables = new Array();
  irw.maxima = new Array();
  irw.colour = new Array();

  var i = 0, j = 0;
  for (i = 0; i < window.nvariables; i++)
  {
    if (window.variables[i].type !=
        Components.interfaces.mozICellMLVariable.BOUND &&
        window.variables[i].type !=
        Components.interfaces.mozICellMLVariable.CONSTANT)
    {
      irw.maxima[j] = 0;
      irw.colour[j] = parseInt(Math.random() * 0xffffff);
      irw.variables[j++] = window.variables[i];
    }
  }
  irw.nvariables = j;
  if (window.integrator == null)
    alert("Internal error: window.integrator is null!");
  irw.integrator = window.integrator;
  irw.maindoc = document;
  irw.mainwin = window;
  irw.bound = window.bound;
  irw.boundunit = window.boundunit;

  if (window.resultsoption == null)
  {
    // We connect before the integrator is started. This allows us to avoid
    // caching any results. Continue the process once the cmgui connection
    // is up.
    window.model.startCmgui(ProxyToMain(irw,
                                        Components.interfaces
                                                  .CmguiIApplicationBootstrap));
  }
}

function CancelIntegration()
{
  if (window.integrator != null)
  {
    ChangeIntegratorButtons(document, true, false);
    window.integrator.suspend();
  }
}

function KillIntegrator()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var statusBar = document.getElementById('status-bar-display');
  statusBar.label = strb.getString("killingIntegrator");
  window.integrator.destroy();
  window.integrator = null;
  ChangeIntegratorButtons(document, false, false);
  statusBar.label = strb.getString("integratorKilled");
}
