function SetupModelParameters()
{
  ParsePageQuery();
  window.model = GetCellMLModel();

  var strb = document.getElementById("mozcellml-string-bundle");
  if (window.translationset == null)
  {
    window.translationset =
      Components.classes["@physiome.org/TranslationSet;1"]
                .createInstance(Components.interfaces.mozITranslationSetLocal);
    window.translationset.bundle = strb.stringBundle;
    window.translationset = window.translationset.QueryInterface(Components.interfaces.mozITranslationSet);
  }

  var de = document.documentElement;
  var title = window.model.title + " - " + de.getAttribute("title");
  de.setAttribute("title", title);

  window.model.setStringBundle(window.translationset);

  //if (window.arguments[1] == null)
  //  window.model.initialise();

  // This should be a pref...
  window.useTreeLayout = GetVariableTreePref();

  var statusBar = GetStatusBar();
  var validity = window.model.checkValidity();
  if (validity == "")
    statusBar.label = strb.getString("modelLoadNoError");
  else
    statusBar.label = strb.getFormattedString("modelInvalidReason", [validity]);

  //if (window.arguments[1] != true)
  //  window.integrator = window.arguments[1];
  var variables = new Object();
  var numvars = new Object();
  window.model.getAllVariables(Components.interfaces.mozICellML.NO_INTERFACE_IN,
                               variables, numvars);
  window.variables = variables.value;
  numvars = numvars.value;
  window.imports = new Array();
  var i = 0;

  var rdfserv = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                          .getService(Components.interfaces.nsIRDFService);
  window.rdfserv = rdfserv;
  var rdfcu = Components.classes['@mozilla.org/rdf/container-utils;1']
                        .getService(Components.interfaces.nsIRDFContainerUtils);
  window.rdfcu = rdfcu;
  var rdfsource = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                            .createInstance(Components.interfaces.nsIRDFDataSource);

  window.rdfsource = rdfsource;
  var rdfvariables = rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#variables");
  var rdfvariable = rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#variable");
  var rdfname = rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#variableName");
  window.rdfname = rdfname;
  var rdftype = rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#variableType");
  var rdfvalue = rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#variableValue");
  window.rdfvalue = rdfvalue;
  var rdfunits = rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#variableUnits");
  var rdfindex = rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#variableIndex");
  var rdfvarprop = rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#variableProperty");

  window.rdfindex = rdfindex;
  window.rdfvalue = rdfvalue;
  window.rdfvarprop = rdfvarprop;
  window.varres = new Array();

  var rdfvar = rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#all-variables");
  var rdfvarcont = rdfcu.MakeSeq(rdfsource, rdfvar);
  window.rdfvarcont = rdfvarcont;
  window.boundcount = 0;
  window.varpristine = new Array();

  for (i = 0; i < numvars;)
  {
    var v = window.variables[i];
    if (v.type == Components.interfaces.mozICellMLVariable.BOUND)
    {
      window.boundcount++;
      window.bound = v.name;
      window.boundunit = v.units;
      //var boundname = document.getElementById("boundname");
      //boundname.value = window.bound + " interval: (";
      //var unitsep = document.getElementById("unitsep");
      //unitsep.value = window.boundunit + " ,";
      //var unitstop = document.getElementById("unitstop");
      //unitstop.value = window.boundunit + ")";
      //var unitgrid = document.getElementById("unitgrid");
      //unitgrid.value = window.boundunit;
    }
    var iv = "";
    try
    {
      iv = v.initialValue;
    }
    catch(e)
    {
      if (e.result != Components.results.NS_ERROR_NOT_AVAILABLE)
      {
        var validity = window.model.checkValidity();
        statusBar.label =
          strb.getFormattedString("modelInvalidReason", [validity]);
      }
    }
    var varRDF = rdfserv.GetAnonymousResource();
    window.varres[i] = varRDF;
    window.varpristine[i] = iv;

    rdfsource.Assert(varRDF, rdfname, rdfserv.GetLiteral(v.name), true);
    rdfsource.Assert(varRDF, rdftype, rdfserv.GetLiteral(TypeToString(v.type)),
                     true);
    rdfsource.Assert(varRDF, rdfvalue, rdfserv.GetLiteral(iv), true);
    rdfsource.Assert(varRDF, rdfunits, rdfserv.GetLiteral(v.units), true);
    rdfsource.Assert(varRDF, rdfindex, rdfserv.GetIntLiteral(i), true);
    window.var_pristine_res = rdfserv.GetLiteral("variable_pristine");
    window.var_modified_res = rdfserv.GetLiteral("variable_modified");
    rdfsource.Assert(varRDF, rdfvarprop, window.var_pristine_res, true);

    if (!window.useTreeLayout)
      rdfvarcont.AppendElement(varRDF);
    else
    {
      var imp = AddOrGetImport(window, v.importNumber);
      var comp = imp.addOrGetComponent(v.componentName);
      comp.AppendElement(varRDF);
    }

    i++;
  }

  var box = document.getElementById("variable-tree");
  box.database.AddDataSource(rdfsource);
  box.builder.rebuild();
  window.nvariables = i;

  //if (window.integrator != null)
  //{
    // XXX we need to synchronise whether an integration is in
    // progress between all the windows that share integrators.
    //ChangeIntegratorButtons(document, true, false);
  //}  
}

function SelectVariable()
{
  var tree = document.getElementById('variable-tree');
  var idx = GetRDFIndexFromTree(window, tree);
  if (idx == -1)
  {
    var tf = document.getElementById("fieldvalue");
    tf.readonly = true;
    tf.value = "";
    var lb = document.getElementById("fieldname");
    lb.value = "";
    return;
  }
  var value = "";
  try
  {
    value = window.variables[idx].initialValue;
  }
  catch (e)
  {
  }
  var name = window.variables[idx].name;
  var tf = document.getElementById("fieldvalue");
  tf.readonly = (window.variables[idx].type ==
                 Components.interfaces.mozICellMLVariable.BOUND) ||
                (window.variables[idx].type ==
                 Components.interfaces.mozICellMLVariable.SIMPLE);
  tf.value = value;
  var lb = document.getElementById("fieldname");
  lb.value = name + ": ";
}

function ChangeInitialValue()
{
  var any_modified = false;

  var tf = document.getElementById("fieldvalue");
  var value = parseFloat(tf.value);
  if (isNaN(value))
    return;
  var tree = document.getElementById('variable-tree');
  var idx = GetRDFIndexFromTree(window, tree);
  if (idx == -1)
    return;
  if ((window.variables[idx].type ==
       Components.interfaces.mozICellMLVariable.BOUND) ||
       (window.variables[idx].type ==
        Components.interfaces.mozICellMLVariable.SIMPLE))
    return;

  window.variables[idx].initialValue = value;

  var ele = window.rdfvarcont.GetElements();
  for (i = 0; i < window.nvariables; i++)
  {
    var el = window.varres[i];
    var idx = window.rdfsource.GetTarget(el, window.rdfindex, true);
    idx = idx.QueryInterface(Components.interfaces.nsIRDFInt).Value;
    try
    {
      var oldValue = window.rdfsource.GetTarget(el, window.rdfvalue, true);
      window.rdfsource.Unassert(el, window.rdfvalue, oldValue);
      var newValue = window.rdfserv.GetLiteral(window.variables[idx].initialValue);
      window.rdfsource.Assert(el, window.rdfvalue, newValue, true);
      var p_on = (window.variables[idx].initialValue == window.varpristine[idx]);
      any_modified = any_modified || (!p_on);
      var a_res = p_on ? window.var_pristine_res : window.var_modified_res;
      var ua_res = window.rdfsource.GetTarget(el, window.rdfvarprop, true);
      window.rdfsource.Unassert(el, window.rdfvarprop, ua_res);
      window.rdfsource.Assert(el, window.rdfvarprop, a_res, true);
    }
    catch (e)
    {
    }
  }

  var m = GetMyMenu();
  if (any_modified)
  {
    m.SetStyle("variable_modified");
    m.parent.SetStyle("variable_modified");
  }
  else
  {
    m.SetStyle("defaultunified");
    m.parent.SetStyle("defaultunified");
  }

  return value;
}
