#!/usr/bin/python
# Copyright (C) 2001,  Christof Meerwald <cmeerw@web.de>
# http://xpcom2corba.sunsite.dk/

# XPCOM-CORBA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

TMPL_proxy_h_file_header = """\
//	-*- C++ -*-
// generated automagically by XPCOM-CORBA
#ifndef @guardname@
#define @guardname@

// include CORBA stub
#include "@CORBAprefix@@basename@@CORBAsuffix@"

// include XPCOM interface declaration
#include "@XPCOMprefix@@basename@@XPCOMsuffix@"

#include <nsCOMPtr.h>
#include <nsMemory.h>
#include <nsString.h>

#include <corba_servant.h>
#include <xpcom_object.h>

#include <xpcom2corba.h>
#include <corba2xpcom.h>
"""

TMPL_proxy_h_file_footer = '#endif'


TMPL_proxy_h_sequence = """\
inline void corba2xpcom(const @corba_name@ &corba_obj, @xpcom_childname@ **xpcom_obj, PRUint32 *length)
{
  *length = corba_obj.length();
  *xpcom_obj = (@xpcom_childname@ *) nsMemory::Alloc(sizeof (@xpcom_childname@) * *length);

  for (PRUint32 i = 0; i < *length; i++)
  {
    ::corba2xpcom(corba_obj[i], (*xpcom_obj) + i);
  }
}
"""

TMPL_proxy_h_sequence_objref = """\
inline void corba2xpcom(const @corba_name@ &corba_obj, @xpcom_childname@ **xpcom_obj, PRUint32 *length)
{
  *length = corba_obj.length();
  *xpcom_obj = (@xpcom_childname@ *) nsMemory::Alloc(sizeof (@xpcom_childname@) * *length);

  for (PRUint32 i = 0; i < *length; i++)
  {
    @xpcom_childname@ _xpcom_child = NULL;
    @convert_child@
    (*xpcom_obj)[i] = _xpcom_child;
  }
}
"""

TMPL_proxy_h_struct_c2x_header = """\
inline void corba2xpcom(const @corba_name@ &corba_obj, @xpcom_name@ **xpcom_obj)
{"""

TMPL_proxy_h_struct_c2x_footer = """\
}
"""

TMPL_proxy_h_struct_proxy = """\
class @intfname@__XPCOM_Proxy
  : public @intfname@
{
 public:
  NS_DECL_ISUPPORTS
  NS_DECL_@uppername@

  explicit @intfname@__XPCOM_Proxy(const @corba_name@ &corba_val);
  virtual ~@intfname@__XPCOM_Proxy();

 private:
  @corba_name@ value_;
};
"""

TMPL_proxy_c_struct_header = """\
NS_IMPL_THREADSAFE_ISUPPORTS1(@intfname@__XPCOM_Proxy, @intfname@)

@intfname@__XPCOM_Proxy::@intfname@__XPCOM_Proxy(const @corba_name@ &corba_val)
 : value_(corba_val)
{
  NS_INIT_ISUPPORTS();
}

@intfname@__XPCOM_Proxy::~@intfname@__XPCOM_Proxy()
{ }
"""


TMPL_proxy_h_intf_header = """
@namespace_begin@
class @proxyname@
  : public virtual POA_@fqname@,
    public @supername@
{
 public:
  explicit @proxyname@(@xpcomintf@ *xpcom_obj);
  virtual ~@proxyname@();

  void add_ref()
  { }

  void release_ref()
  { }
"""

TMPL_proxy_h_intf_footer = """
  virtual const char **_operation_names() const;
  static const char *_operation_names_[];

  virtual const char *_repository_id() const;
  static const char *_repository_id_;
  
  virtual const char **_base_repository_ids() const;
  static const char *_base_repository_ids_[];

 private:
  nsCOMPtr<@xpcomintf@> xpcom_obj_;
};
@namespace_end@
"""

TMPL_proxy_h_operation = """\
virtual @retspec@ @name@(@paramspec@);\
"""


TMPL_proxy_c_file_header = """\
//	-*- C++ -*-
// generated automagically by XPCOM-CORBA
#include "@basename@_xpcom.h"
#include "@basename@_corba.h"

#include <corba_proxy.h>
#include <corba_poa.h>
"""

TMPL_proxy_c_intf = """
CORBA_Proxy__@fqname@::@proxyname@(@xpcomintf@ *xpcom_obj)
  : @supername@(xpcom_obj),
    xpcom_obj_(xpcom_obj)
{
}

CORBA_Proxy__@fqname@::~@proxyname@()
{
}


@namespace_begin@
class @proxyname@__Factory
  : public CORBA_Proxy_Factory
{
 public:
  explicit @proxyname@__Factory(const char *repo_id)
    : CORBA_Proxy_Factory(repo_id, this)
  { }

  virtual ~@proxyname@__Factory()
  { }


  virtual CORBA_Proxy_Servant *GetProxy(nsISupports *obj) const;
};

const @proxyname@__Factory
  the__@proxyname@__Factory(CORBA_Proxy__@fqname@::_repository_id_);
@namespace_end@

CORBA_Proxy_Servant *
CORBA_Proxy__@fqname@__Factory::GetProxy(nsISupports *obj) const
{
  nsCOMPtr<@xpcomintf@> narrowed_obj = do_QueryInterface(obj);
  if (narrowed_obj)
  {
    return static_cast<CORBA_Proxy_Servant *>(new CORBA_Proxy__@fqname@(narrowed_obj));
  }
  else
  {
    return NULL;
  }
}


const char *CORBA_Proxy__@fqname@::_repository_id() const
{
  return _repository_id_;
}

const char *CORBA_Proxy__@fqname@::_repository_id_ = "@repoid@";
"""

TMPL_proxy_c_intf_opnames_header = """\
const char **CORBA_Proxy__@fqname@::_operation_names() const
{
  return _operation_names_;
}

const char *CORBA_Proxy__@fqname@::_operation_names_[] =
{"""

TMPL_proxy_c_intf_opnames_footer = """\
  NULL
};
"""

TMPL_proxy_c_intf_baserepoids_header = """\
const char **CORBA_Proxy__@fqname@::_base_repository_ids() const
{
  return _base_repository_ids_;
}

const char *CORBA_Proxy__@fqname@::_base_repository_ids_[] =
{"""

TMPL_proxy_c_intf_baserepoids_footer = """\
  NULL
};
"""


TMPL_proxy_c_operation_header = """\
@retspec@ CORBA_Proxy__@fqname@::@name@(@paramspec@)
{
  nsresult _result;
"""

TMPL_proxy_c_operation_stringout = """\
@stringvar@ = (char *) nsMemory::Clone(@stringvar@, sizeof (char) *
                                         (strlen(@stringvar@) + 1));
if (@stringvar@ == 0)
{
  _result = NS_ERROR_OUT_OF_MEMORY;
}
"""

TMPL_proxy_c_operation_footer = """
}
"""
