/*	-*- C++ -*-
 * Copyright (C) 2001,  Christof Meerwald <cmeerw@web.de>
 * http://xpcom2corba.sunsite.dk/
 */

/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License
 * at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and
 * limitations under the License.
 *
 * The Original Code is the XPCOM-CORBA Bridge package.
 *
 * The Initial Developer of the Original Code is Christof Meerwald.
 * Portions created by Christof Meerwald are Copyright (C) 2001
 * Christof Meerwald.  All Rights Reserved.
 *
 * Contributor(s): Christof Meerwald <cmeerw@web.de> (original author)
 */
#ifndef HEADER__CORBA_POA__INCLUDED
#define HEADER__CORBA_POA__INCLUDED

#include "xpcorba_config.h"

#include <CORBA.h>

#include "corba_locator.h"
#include "corba_proxy.h"

// forward declarations
class nsISupports;


class CORBA_POA
{
 public:
  CORBA_POA();

  ~CORBA_POA();


  void init(int argc, char *argv[]);

  template<class TypeXPCOM, class TypeCORBA>
  typename TypeCORBA::_ptr_type register_object(TypeXPCOM *xpcom_obj, TypeCORBA *dummy)
  {
    if (xpcom_obj)
    {
      const char *const repoid = TypeCORBA::_PD_repoId;
      CORBA_Proxy_Object *corba_proxy_obj = new CORBA_Proxy_Object(xpcom_obj);
      corba_proxy_obj->add_repoid(repoid);

      const char *const oid_str =
	proxy_locator_->register_proxy(corba_proxy_obj);
      PortableServer::ObjectId_var oid =
	PortableServer::string_to_ObjectId(oid_str);

      corba_proxy_obj->oid(oid_str);

      // the use of _PD_repoId is omniORB specific
      CORBA::Object_var proxy_obj =
	xpcom_poa_->create_reference_with_id(oid, TypeCORBA::_PD_repoId);
      return TypeCORBA::_narrow(proxy_obj);
    }
    else
    {
      return TypeCORBA::_nil();
    }
  }

  inline void unregister_object(const char *oid_str)
  {
    proxy_locator_->unregister_proxy(oid_str);
  }


 private:
  CORBA::ORB_var orb_;
  PortableServer::POA_var xpcom_poa_;
  CORBA_Proxy_Locator *proxy_locator_;
};


extern CORBA_POA *the_corba_poa;

#endif
